@@ -19,11 +19,10 @@ from __future__ import division
 from __future__ import unicode_literals
 
 from apitools.base.py import list_pager
-
 from googlecloudsdk.api_lib.compute import utils
 from googlecloudsdk.api_lib.util import apis
 from googlecloudsdk.api_lib.util import waiter
-from googlecloudsdk.command_lib.filestore import locations_util
+from googlecloudsdk.command_lib.filestore.backups import util as backup_util
 from googlecloudsdk.command_lib.filestore.snapshots import util as snapshot_util
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
@@ -62,6 +61,10 @@ class Error(exceptions.Error):
   """Base class for exceptions in this module."""
 
 
+class InvalidArgumentError(Error):
+  """Raised when command line argument constraints are violated."""
+
+
 class InvalidCapacityError(Error):
   """Raised when an invalid capacity value is provided."""
 
@@ -127,6 +130,12 @@ class FilestoreClient(object):
         name=snapshot_ref.RelativeName())
     return self.client.projects_locations_snapshots.Get(request)
 
+  def GetBackup(self, backup_ref):
+    """Get Cloud Filestore backup information."""
+    request = self.messages.FileProjectsLocationsBackupsGetRequest(
+        name=backup_ref.RelativeName())
+    return self.client.projects_locations_backups.Get(request)
+
   def DeleteInstance(self, instance_ref, async_):
     """Delete an existing Cloud Filestore instance."""
     request = self.messages.FileProjectsLocationsInstancesDeleteRequest(
@@ -168,6 +177,19 @@ class FilestoreClient(object):
             self.client.projects_locations_operations), operation_ref,
         'Waiting for [{0}] to finish'.format(operation_ref.Name()))
 
+  def CancelOperation(self, operation_ref):
+    """Cancels a long-running operation.
+
+    Args:
+      operation_ref: the operation reference.
+
+    Returns:
+      Empty response message.
+    """
+    request = self.messages.FileProjectsLocationsOperationsCancelRequest(
+        name=operation_ref.RelativeName())
+    return self.client.projects_locations_operations.Cancel(request)
+
   def CreateInstance(self, instance_ref, async_, config):
     """Create a Cloud Filestore instance."""
     request = self.messages.FileProjectsLocationsInstancesCreateRequest(
@@ -182,8 +204,7 @@ class FilestoreClient(object):
     return self.WaitForOperation(operation_ref)
 
   def GetLocation(self, location_ref):
-    request = self.messages.FileProjectsLocationsGetRequest(
-        name=location_ref)
+    request = self.messages.FileProjectsLocationsGetRequest(name=location_ref)
     return self.client.projects_locations.Get(request)
 
   def ListLocations(self, project_ref, limit=None):
@@ -221,7 +242,8 @@ class FilestoreClient(object):
     gb_in_one_tb = 1 << 10
     minimum_values = {
         self.messages.Instance.TierValueValuesEnum.STANDARD: gb_in_one_tb,
-        self.messages.Instance.TierValueValuesEnum.PREMIUM: 2.5 * gb_in_one_tb}
+        self.messages.Instance.TierValueValuesEnum.PREMIUM: 2.5 * gb_in_one_tb
+    }
     minimum = minimum_values.get(instance_tier, 0)
     if capacity_gb < minimum:
       raise InvalidCapacityError(
@@ -234,8 +256,14 @@ class FilestoreClient(object):
       if file_share.capacityGb:
         self._ValidateFileShare(instance.tier, file_share.capacityGb)
 
-  def ParseFilestoreConfig(self, tier=None, description=None, file_share=None,
-                           network=None, labels=None, zone=None):
+  def ParseFilestoreConfig(self,
+                           tier=None,
+                           description=None,
+                           file_share=None,
+                           network=None,
+                           labels=None,
+                           zone=None,
+                           nfs_export_options=None):
     """Parses the command line arguments for Create into a config.
 
     Args:
@@ -245,6 +273,7 @@ class FilestoreClient(object):
       network: The network for the instance.
       labels: The parsed labels value.
       zone: The parsed zone of the instance.
+      nfs_export_options: the nfs export options for the file share.
 
     Returns:
       the configuration that will be used as the request body for creating a
@@ -257,6 +286,8 @@ class FilestoreClient(object):
 
     if description:
       instance.description = description
+    if nfs_export_options:
+      file_share['nfs_export_options'] = nfs_export_options
 
     self._adapter.ParseFileShareIntoInstance(instance, file_share, zone)
 
@@ -269,8 +300,11 @@ class FilestoreClient(object):
       instance.networks.append(network_config)
     return instance
 
-  def ParseUpdatedInstanceConfig(self, instance_config, description=None,
-                                 labels=None, file_share=None):
+  def ParseUpdatedInstanceConfig(self,
+                                 instance_config,
+                                 description=None,
+                                 labels=None,
+                                 file_share=None):
     """Parses updates into an instance config.
 
     Args:
@@ -287,7 +321,9 @@ class FilestoreClient(object):
       The instance message.
     """
     instance = self._adapter.ParseUpdatedInstanceConfig(
-        instance_config, description=description, labels=labels,
+        instance_config,
+        description=description,
+        labels=labels,
         file_share=file_share)
     return instance
 
@@ -303,14 +339,56 @@ class FilestoreClient(object):
     Returns:
       an Operation or Instance message.
     """
-    update_op = self._adapter.UpdateInstance(
-        instance_ref, instance_config, update_mask)
+    update_op = self._adapter.UpdateInstance(instance_ref, instance_config,
+                                             update_mask)
     if async_:
       return update_op
     operation_ref = resources.REGISTRY.ParseRelativeName(
         update_op.name, collection=OPERATIONS_COLLECTION)
     return self.WaitForOperation(operation_ref)
 
+  @staticmethod
+  def MakeNFSExportOptionsMsg(messages, nfs_export_options):
+    """Creates an NfsExportOptions message.
+
+    Args:
+      messages: The messages module.
+      nfs_export_options: list, containing NfsExportOptions dictionary.
+
+    Returns:
+      File share message populated with values, filled with defaults.
+      In case no nfs export options are provided we rely on the API to apply a
+      default.
+    """
+    read_write = 'READ_WRITE'
+    root_squash = 'ROOT_SQUASH'
+    no_root_squash = 'NO_ROOT_SQUASH'
+    anonimous_uid = 65534
+    anonimous_gid = 65534
+    nfs_export_configs = []
+
+    if nfs_export_options is None:
+      return []
+    for nfs_export_option in nfs_export_options:
+      access_mode = messages.NfsExportOptions.AccessModeValueValuesEnum.lookup_by_name(
+          nfs_export_option.get('access-mode', read_write))
+      squash_mode = messages.NfsExportOptions.SquashModeValueValuesEnum.lookup_by_name(
+          nfs_export_option.get('squash-mode', no_root_squash))
+      if nfs_export_option.get('squash-mode', None) == root_squash:
+        anon_uid = nfs_export_option.get('anon_uid', anonimous_uid)
+        anon_gid = nfs_export_option.get('anon_gid', anonimous_gid)
+      else:
+        anon_gid = None
+        anon_uid = None
+      nfs_export_config = messages.NfsExportOptions(
+          ipRanges=nfs_export_option.get('ip-ranges'),
+          anonUid=anon_uid,
+          anonGid=anon_gid,
+          accessMode=access_mode,
+          squashMode=squash_mode)
+      nfs_export_configs.append(nfs_export_config)
+    return nfs_export_configs
+
 
 class AlphaFilestoreAdapter(object):
   """Adapter for the alpha filestore API."""
@@ -319,64 +397,105 @@ class AlphaFilestoreAdapter(object):
     self.client = GetClient(version=ALPHA_API_VERSION)
     self.messages = GetMessages(version=ALPHA_API_VERSION)
 
-  def ParseFileShareIntoInstance(self, instance, file_share,
+  def ParseFileShareIntoInstance(self,
+                                 instance,
+                                 file_share,
                                  instance_zone=None):
-    """Parse specified file share configs into an instance message."""
+    """Parse specified file share configs into an instance message.
+
+    Args:
+      instance: The Filestore instance object.
+      file_share: File share configuration.
+      instance_zone: The instance zone.
+
+    Raises:
+      InvalidArgumentError: If file_share argument constraints are violated.
+
+    """
     if instance.fileShares is None:
       instance.fileShares = []
     if file_share:
       source_snapshot = None
+      source_backup = None
+      location = None
+
+      # Deduplicate file shares with the same name.
+      instance.fileShares = [
+          fs for fs in instance.fileShares if fs.name != file_share.get('name')
+      ]
       if 'source-snapshot' in file_share:
         project = properties.VALUES.core.project.Get(required=True)
-        location = (file_share.get('source-snapshot-region') or
-                    locations_util.GetRegionFromZone(instance_zone))
+        location = (file_share.get('source-snapshot-region') or instance_zone)
         source_snapshot = snapshot_util.SNAPSHOT_NAME_TEMPLATE.format(
             project, location, file_share.get('source-snapshot'))
+      if 'source-backup' in file_share:
+        project = properties.VALUES.core.project.Get(required=True)
+        location = file_share.get('source-backup-region')
+        source_backup = backup_util.BACKUP_NAME_TEMPLATE.format(
+            project, location, file_share.get('source-backup'))
+
+      if None not in [source_snapshot, source_backup]:
+        raise InvalidArgumentError(
+            "At most one of ['source-snapshot', 'source-backup'] may be specified."
+        )
+      if source_backup is not None and location is None:
+        raise InvalidArgumentError(
+            "If 'source-backup' is specified, 'source-backup-region' must also be specified."
+        )
+
+      nfs_export_options = FilestoreClient.MakeNFSExportOptionsMsg(
+          self.messages, file_share.get('nfs-export-options', []))
       file_share_config = self.messages.FileShareConfig(
           name=file_share.get('name'),
           capacityGb=utils.BytesToGb(file_share.get('capacity')),
-          sourceSnapshot=source_snapshot)
+          sourceSnapshot=source_snapshot,
+          sourceBackup=source_backup,
+          nfsExportOptions=nfs_export_options)
       instance.fileShares.append(file_share_config)
 
   def FileSharesFromInstance(self, instance):
     """Get file share configs from instance message."""
     return instance.fileShares
 
-  def UpdateInstance(self, instance_ref, instance_config, update_mask):
-    # Not supported by alpha API.
-    raise NotImplementedError
-
-  def ParseUpdatedInstanceConfig(self, instance_config, description=None,
-                                 labels=None, file_shares=None):
-    # Not supported by alpha API.
-    raise NotImplementedError
-
-
-class BetaFilestoreAdapter(object):
-  """Adapter for the beta filestore API."""
-
-  def __init__(self):
-    self.client = GetClient(version=BETA_API_VERSION)
-    self.messages = GetMessages(version=BETA_API_VERSION)
-
-  def ParseFileShareIntoInstance(self, instance, file_share,
-                                 instance_zone=None):
-    """Parse specified file share configs into an instance message."""
-    del instance_zone  # Unused.
-    if instance.fileShares is None:
-      instance.fileShares = []
+  def ParseUpdatedInstanceConfig(self,
+                                 instance_config,
+                                 description=None,
+                                 labels=None,
+                                 file_share=None):
+    """Parse update information into an updated Instance message."""
+    if description:
+      instance_config.description = description
+    if labels:
+      instance_config.labels = labels
     if file_share:
-      # Deduplicate file shares with the same name.
-      instance.fileShares = [fs for fs in instance.fileShares
-                             if fs.name != file_share.get('name')]
-      file_share_config = self.messages.FileShareConfig(
-          name=file_share.get('name'),
-          capacityGb=utils.BytesToGb(file_share.get('capacity')))
-      instance.fileShares.append(file_share_config)
+      self.ValidateFileShareForUpdate(instance_config, file_share)
+      self.ParseFileShareIntoInstance(instance_config, file_share)
+    return instance_config
 
-  def FileSharesFromInstance(self, instance):
-    """Get fileshare configs from instance message."""
-    return instance.fileShares
+  def ValidateFileShareForUpdate(self, instance_config, file_share):
+    """Validate the updated file share configuration.
+
+    The new config must have the same name as the existing config and a larger
+    size than the existing capacity.
+
+    Args:
+      instance_config: Instance message for existing instance.
+      file_share: dict with keys 'name' and 'capacity'.
+
+    Raises:
+      InvalidNameError: If the names don't match.
+      ValueError: If the instance doesn't have an existing file share.
+    """
+    existing = self.FileSharesFromInstance(instance_config)
+    if not existing:
+      # This should never happen because all instances have one file share.
+      raise ValueError('Existing instance does not have file shares configured')
+    existing_file_share = existing[0]
+    if existing_file_share.name != file_share.get('name'):
+      raise InvalidNameError(
+          'Must update an existing file share. Existing file share is named '
+          '[{}]. Requested update had name [{}].'.format(
+              existing_file_share.name, file_share.get('name')))
 
   def UpdateInstance(self, instance_ref, instance_config, update_mask):
     """Send a Patch request for the Cloud Filestore instance."""
@@ -387,17 +506,82 @@ class BetaFilestoreAdapter(object):
     update_op = self.client.projects_locations_instances.Patch(update_request)
     return update_op
 
-  def ParseUpdatedInstanceConfig(self, instance_config, description=None,
-                                 labels=None, file_share=None):
-    """Parse update information into an updated Instance message."""
-    if description:
-      instance_config.description = description
-    if labels:
-      instance_config.labels = labels
+
+class BetaFilestoreAdapter(AlphaFilestoreAdapter):
+  """Adapter for the beta filestore API."""
+
+  def __init__(self):
+    super(BetaFilestoreAdapter, self).__init__()
+    self.client = GetClient(version=BETA_API_VERSION)
+    self.messages = GetMessages(version=BETA_API_VERSION)
+
+  def ParseFileShareIntoInstance(self,
+                                 instance,
+                                 file_share,
+                                 instance_zone=None):
+    """Parse specified file share configs into an instance message."""
+    del instance_zone  # Unused.
+    if instance.fileShares is None:
+      instance.fileShares = []
     if file_share:
-      self.ValidateFileShareForUpdate(instance_config, file_share)
-      self.ParseFileShareIntoInstance(instance_config, file_share)
-    return instance_config
+      source_backup = None
+      location = None
+
+      # Deduplicate file shares with the same name.
+      instance.fileShares = [
+          fs for fs in instance.fileShares if fs.name != file_share.get('name')
+      ]
+      if 'source-backup' in file_share:
+        project = properties.VALUES.core.project.Get(required=True)
+        location = file_share.get('source-backup-region')
+        if location is None:
+          raise InvalidArgumentError(
+              "If 'source-backup' is specified, 'source-backup-region' must also "
+              "be specified.")
+        source_backup = backup_util.BACKUP_NAME_TEMPLATE.format(
+            project, location, file_share.get('source-backup'))
+
+      nfs_export_options = FilestoreClient.MakeNFSExportOptionsMsg(
+          self.messages, file_share.get('nfs-export-options', []))
+      file_share_config = self.messages.FileShareConfig(
+          name=file_share.get('name'),
+          capacityGb=utils.BytesToGb(file_share.get('capacity')),
+          sourceBackup=source_backup,
+          nfsExportOptions=nfs_export_options)
+      instance.fileShares.append(file_share_config)
+
+  def FileSharesFromInstance(self, instance):
+    """Get fileshare configs from instance message."""
+    return instance.fileShares
+
+
+class FilestoreAdapter(BetaFilestoreAdapter):
+  """Adapter for the filestore v1 API."""
+
+  def __init__(self):
+    super(FilestoreAdapter, self).__init__()
+    self.client = GetClient(version=V1_API_VERSION)
+    self.messages = GetMessages(version=V1_API_VERSION)
+
+  def ParseFileShareIntoInstance(self, instance, file_share,
+                                 instance_zone=None):
+    """Parse specified file share configs into an instance message."""
+    del instance_zone  # Unused.
+    if instance.fileShares is None:
+      instance.fileShares = []
+    if file_share:
+      # Deduplicate file shares with the same name.
+      instance.fileShares = [
+          fs for fs in instance.fileShares if fs.name != file_share.get('name')
+      ]
+
+      nfs_export_options = FilestoreClient.MakeNFSExportOptionsMsg(
+          self.messages, file_share.get('nfs-export-options', []))
+      file_share_config = self.messages.FileShareConfig(
+          name=file_share.get('name'),
+          capacityGb=utils.BytesToGb(file_share.get('capacity')),
+          nfsExportOptions=nfs_export_options)
+      instance.fileShares.append(file_share_config)
 
   def ValidateFileShareForUpdate(self, instance_config, file_share):
     """Validate the updated file share configuration.
@@ -421,26 +605,17 @@ class BetaFilestoreAdapter(object):
     existing_file_share = existing[0]
     if existing_file_share.name != file_share.get('name'):
       raise InvalidNameError(
-          'Must resize an existing file share. Existing file share is named '
+          'Must update an existing file share. Existing file share is named '
           '[{}]. Requested update had name [{}].'.format(
               existing_file_share.name, file_share.get('name')))
     new_capacity = utils.BytesToGb(file_share.get('capacity'))
     if not existing_file_share.capacityGb < new_capacity:
       raise InvalidCapacityError(
-          'Must resize the file share to a larger capacity. Existing capacity: '
+          'Must update the file share to a larger capacity. Existing capacity: '
           '[{}]. New capacity requested: [{}].'.format(
               existing_file_share.capacityGb, new_capacity))
 
 
-class FilestoreAdapter(BetaFilestoreAdapter):
-  """Adapter for the filestore v1 API."""
-
-  def __init__(self):
-    super(FilestoreAdapter, self).__init__()
-    self.client = GetClient(version=V1_API_VERSION)
-    self.messages = GetMessages(version=V1_API_VERSION)
-
-
 def GetFilestoreRegistry(api_version=V1_API_VERSION):
   registry = resources.REGISTRY.Clone()
   registry.RegisterApiByName(API_NAME, api_version=api_version)
