@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2013 Google LLC. All Rights Reserved.
+# Copyright 2020 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -117,7 +117,7 @@ class ReleaseTrack(object):
       'This command is currently in ALPHA and may change without notice. '
       'If this command fails with API permission errors despite specifying '
       'the right project, you may be trying to access an API with '
-      'an invitation-only early access whitelist.')
+      'an invitation-only early access allowlist.')
   _ALL = [GA, BETA, ALPHA]
 
   @staticmethod
@@ -419,7 +419,12 @@ URI_FLAG = Argument(
     action='store_true',
     require_coverage_in_tests=False,
     category=LIST_COMMAND_FLAGS,
-    help='Print a list of resource URIs instead of the default output.')
+    help="""\
+    Print a list of resource URIs instead of the default output, and change the
+    command output to a list of URIs. If this flag is used with *--format*,
+    the formatting is applied on this URI list. To display URIs alongside other
+    keys instead, use the *uri()* transform.
+    """)
 
 # Binary Command Flags
 BINARY_BACKED_COMMAND_FLAGS = 'BINARY BACKED COMMAND'
@@ -664,6 +669,10 @@ class ExportCommand(six.with_metaclass(abc.ABCMeta, Command)):
   """A command that outputs one resource to file in yaml format."""
 
 
+class DeclarativeCommand(six.with_metaclass(abc.ABCMeta, Command)):
+  """Command class for managing gcp resources as YAML/JSON files."""
+
+
 class BinaryBackedCommand(six.with_metaclass(abc.ABCMeta, Command)):
   """A command that wraps a BinaryBackedOperation."""
 
@@ -941,24 +950,58 @@ def EnableUserProjectQuota():
       properties.VALUES.billing.CURRENT_PROJECT)
 
 
-def AllowGoogleAuth():
-  """Allows the command group to use google auth for authentication.
+def EnableUserProjectQuotaWithFallback():
+  """Tries the current project and fall back to the legacy mode."""
+  properties.VALUES.billing.quota_project.Set(
+      properties.VALUES.billing.CURRENT_PROJECT_WITH_FALLBACK)
+
+
+def UserProjectQuotaWithFallbackEnabled():
+  """Returns if the CURRENT_PROJECT_WITH_FALLBACK mode is enabled."""
+  return properties.VALUES.billing.quota_project.Get(
+  ) == properties.VALUES.billing.CURRENT_PROJECT_WITH_FALLBACK
+
+
+def OptOutRequests():
+  """Opts the command group out of using requests to make HTTP requests.
 
   Call this function in the Filter method of the command group
-  to enable google-auth.
+  to disable requests.
   """
-  properties.VALUES.auth.google_auth_allowed.Set(True)
+  properties.VALUES.transport.opt_out_requests.Set(True)
+
+
+def UseRequests():
+  """Returns True if using requests to make HTTP requests.
+
+  transport/disable_requests_override is a global switch to turn off requests in
+  case support is buggy. transport/opt_out_requests is an internal property
+  to opt surfaces out of requests.
+  """
+
+  return (UseGoogleAuth() and
+          not properties.VALUES.transport.opt_out_requests.GetBool() and
+          not properties.VALUES.transport.disable_requests_override.GetBool())
+
+
+def OptOutGoogleAuth():
+  """Opt-out the command group to use google auth for authentication.
+
+  Call this function in the Filter method of the command group
+  to opt-out google-auth.
+  """
+  properties.VALUES.auth.opt_out_google_auth.Set(True)
 
 
 def UseGoogleAuth():
   """Returns True if using google-auth to authenticate the http request.
 
   auth/disable_load_google_auth is a global switch to turn off google-auth in
-  case google-auth is buggy. auth/google_auth_allowed is an internal property
-  to opt-in surface.
+  case google-auth is crashing. auth/opt_out_google_auth is an internal property
+  to opt-out a surface.
   """
-  return (properties.VALUES.auth.google_auth_allowed.GetBool() and
-          not properties.VALUES.auth.disable_load_google_auth.GetBool())
+  return not (properties.VALUES.auth.opt_out_google_auth.GetBool() or
+              properties.VALUES.auth.disable_load_google_auth.GetBool())
 
 
 def LogCommand(prog, args):
