@@ -174,7 +174,8 @@ class Settings(DataObject):
       service_name = args.service_name
     else:
       dir_name = os.path.basename(files.GetCWD())
-      service_name = dir_name.replace('_', '-')
+      # Service names may not include _ and upper case characters.
+      service_name = dir_name.replace('_', '-').lower()
 
     if not args.IsSpecified('image'):
       if project_name:
@@ -183,6 +184,11 @@ class Settings(DataObject):
       else:
         image = service_name
 
+      # Image names cannot have upper case characters. If the image name is
+      # autogenerated, then make sure the image name is lower case. If the
+      # user enters an image name manually, it's OK to stop the user and tell
+      # them about the illegal image name.
+      image = image.lower()
     else:
       image = args.image
 
@@ -649,10 +655,6 @@ class SecretInfo(object):
                  '/local_development_service_account.json')
 
 
-class ADCMissingError(exceptions.Error):
-  """Application Default Credential is missing."""
-
-
 def GetServiceAccountSecret(account_name):
   """Get a service account secret file as text.
 
@@ -671,14 +673,9 @@ def GetUserCredential():
 
   Returns:
     Text version of the user's application default credential.
-  Raises:
-    ADCMissingError: Application Default Credential is not present.
   """
-  adc = auth_util.GetADCAsJson()
-  if not adc:
-    raise ADCMissingError('No user credential present. Run '
-                          '`$ gcloud auth application-default login`.')
-  return json.dumps(adc)
+  auth_util.AssertADCExists()
+  return json.dumps(auth_util.GetADCAsJson())
 
 
 class CredentialGenerator(KubeConfigGenerator):
