@@ -19,7 +19,6 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
-import collections
 import enum
 import os
 import re
@@ -28,24 +27,27 @@ from apitools.base.py import exceptions as apitools_exceptions
 from googlecloudsdk.api_lib.container import kubeconfig
 from googlecloudsdk.api_lib.run import global_methods
 from googlecloudsdk.api_lib.run import revision
+from googlecloudsdk.api_lib.run import service
 from googlecloudsdk.api_lib.run import traffic
 from googlecloudsdk.api_lib.services import enable_api
 from googlecloudsdk.api_lib.services import exceptions as services_exceptions
 from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
-from googlecloudsdk.calliope import exceptions as calliope_exceptions
-from googlecloudsdk.command_lib.functions.deploy import env_vars_util
+from googlecloudsdk.command_lib.functions.v1.deploy import env_vars_util
 from googlecloudsdk.command_lib.run import config_changes
 from googlecloudsdk.command_lib.run import exceptions as serverless_exceptions
+from googlecloudsdk.command_lib.run import platforms
 from googlecloudsdk.command_lib.run import pretty_print
+from googlecloudsdk.command_lib.run import resource_args
 from googlecloudsdk.command_lib.util.args import labels_util
 from googlecloudsdk.command_lib.util.args import map_util
 from googlecloudsdk.command_lib.util.args import repeated
+from googlecloudsdk.command_lib.util.concepts import concept_parsers
+from googlecloudsdk.core import config
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
-from googlecloudsdk.core import resources
 from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.util import encoding
 from googlecloudsdk.core.util import files
@@ -55,27 +57,16 @@ _VISIBILITY_MODES = {
     'external': 'Visible from outside the cluster.',
 }
 
-PLATFORM_MANAGED = 'managed'
-PLATFORM_GKE = 'gke'
-PLATFORM_KUBERNETES = 'kubernetes'
-
-_PLATFORMS = collections.OrderedDict([
-    (PLATFORM_MANAGED, 'Fully managed version of Cloud Run. '
-     'Use with the `--region` flag or set the [run/region] property '
-     'to specify a Cloud Run region.'),
-    (PLATFORM_GKE, 'Cloud Run for Anthos on Google Cloud. '
-     'Use with the `--cluster` and `--cluster-location` flags or set the '
-     '[run/cluster] and [run/cluster_location] properties to specify a '
-     'cluster in a given zone.'),
-    (PLATFORM_KUBERNETES, 'Use a Knative-compatible kubernetes cluster. '
-     'Use with the `--kubeconfig` and `--context` flags to specify a '
-     'kubeconfig file and the context for connecting.'),
-])
-
-_PLATFORM_SHORT_DESCRIPTIONS = {
-    PLATFORM_MANAGED: 'Cloud Run (fully managed)',
-    PLATFORM_GKE: 'Cloud Run for Anthos deployed on Google Cloud',
-    PLATFORM_KUBERNETES: 'Cloud Run for Anthos deployed on VMware',
+_INGRESS_MODES = {
+    'all': 'Inbound requests from all sources are allowed.',
+    'internal':
+        'For Cloud Run (fully managed), only inbound requests from VPC networks'
+        ' in the same project are allowed. For Cloud Run for Anthos, only '
+        'inbound requests from the same cluster are allowed.',
+    'internal-and-cloud-load-balancing':
+        'Only supported for Cloud Run (fully managed). Only inbound requests'
+        ' from VPC networks in the same project or from Google Cloud Load '
+        'Balancing are allowed.'
 }
 
 _DEFAULT_KUBECONFIG_PATH = '~/.kube/config'
@@ -83,10 +74,6 @@ _DEFAULT_KUBECONFIG_PATH = '~/.kube/config'
 _FIFTEEN_MINUTES = 15 * 60
 
 
-class ArgumentError(exceptions.Error):
-  pass
-
-
 class KubeconfigError(exceptions.Error):
   pass
 
@@ -96,11 +83,11 @@ class Product(enum.Enum):
   EVENTS = 'Events'
 
 
-def AddImageArg(parser):
+def AddImageArg(parser, required=True):
   """Add an image resource arg."""
   parser.add_argument(
       '--image',
-      required=True,
+      required=required,
       help='Name of the container image to deploy (e.g. '
       '`gcr.io/cloudrun/hello:latest`).')
 
@@ -156,8 +143,9 @@ def GetManagedArgGroup(parser):
   return _GetOrAddArgGroup(
       parser,
       _ARG_GROUP_HELP_TEXT.format(
-          platform='`--platform={}`'.format(PLATFORM_MANAGED),
-          platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_MANAGED]))
+          platform='`--platform={}`'.format(platforms.PLATFORM_MANAGED),
+          platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+              platforms.PLATFORM_MANAGED]))
 
 
 def GetGkeArgGroup(parser):
@@ -165,8 +153,9 @@ def GetGkeArgGroup(parser):
   return _GetOrAddArgGroup(
       parser,
       _ARG_GROUP_HELP_TEXT.format(
-          platform='`--platform={}`'.format(PLATFORM_GKE),
-          platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_GKE]))
+          platform='`--platform={}`'.format(platforms.PLATFORM_GKE),
+          platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+              platforms.PLATFORM_GKE]))
 
 
 def GetKubernetesArgGroup(parser):
@@ -174,8 +163,9 @@ def GetKubernetesArgGroup(parser):
   return _GetOrAddArgGroup(
       parser,
       _ARG_GROUP_HELP_TEXT.format(
-          platform='`--platform={}`'.format(PLATFORM_KUBERNETES),
-          platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_KUBERNETES]))
+          platform='`--platform={}`'.format(platforms.PLATFORM_KUBERNETES),
+          platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+              platforms.PLATFORM_KUBERNETES]))
 
 
 def GetClusterArgGroup(parser):
@@ -184,10 +174,40 @@ def GetClusterArgGroup(parser):
       parser,
       _ARG_GROUP_HELP_TEXT.format(
           platform='`--platform={}` or `--platform={}`'.format(
-              PLATFORM_GKE, PLATFORM_KUBERNETES),
+              platforms.PLATFORM_GKE, platforms.PLATFORM_KUBERNETES),
           platform_desc='{} or {}'.format(
-              _PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_GKE],
-              _PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_KUBERNETES])))
+              platforms.PLATFORM_SHORT_DESCRIPTIONS[platforms.PLATFORM_GKE],
+              platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                  platforms.PLATFORM_KUBERNETES])))
+
+
+def AddPlatformAndLocationFlags(parser, managed_only=False, anthos_only=False):
+  """Adds flags used to determine the platform and the location of resource."""
+  assert not (managed_only and anthos_only)
+  AddPlatformArg(parser, managed_only, anthos_only)
+
+  if managed_only:
+    AddRegionArg(parser)
+    return None
+
+  # When multiple platforms are supported, add a arg group that covers the
+  # various ways to specify region/zone/cluster.
+  platform_helpers_group = parser.add_mutually_exclusive_group(
+      help='Arguments to locate resources, depending on the platform used.')
+
+  if not anthos_only:
+    # Add --region flag
+    managed_group = GetManagedArgGroup(platform_helpers_group)
+    AddRegionArg(managed_group)
+
+  # Add --cluster and --cluster-location (plus properties)
+  gke_group = GetGkeArgGroup(platform_helpers_group)
+  concept_parsers.ConceptParser([resource_args.CLUSTER_PRESENTATION
+                                ]).AddToParser(gke_group)
+
+  # Add --kubeconfig and --context
+  kubernetes_group = GetKubernetesArgGroup(platform_helpers_group)
+  AddKubeconfigFlags(kubernetes_group)
 
 
 def AddAllowUnauthenticatedFlag(parser):
@@ -204,14 +224,32 @@ def AddAsyncFlag(parser):
   base.ASYNC_FLAG.AddToParser(parser)
 
 
-def AddEndpointVisibilityEnum(parser):
+def AddEndpointVisibilityEnum(parser, deprecated=False):
   """Add the --connectivity=[external|internal] flag."""
+  action = None
+  if deprecated:
+    action = actions.DeprecationAction(
+        '--connectivity',
+        warn='The {flag_name} flag is deprecated and will be removed in an '
+        'upcoming release. Please use the --ingress flag instead.')
   parser.add_argument(
       '--connectivity',
       choices=_VISIBILITY_MODES,
       help=('Defaults to \'external\'. If \'external\', the service can be '
             'invoked through the internet, in addition to through the cluster '
-            'network.'))
+            'network.'),
+      action=action)
+
+
+def AddIngressFlag(parser):
+  """Adds the --ingress flag."""
+  parser.add_argument(
+      '--ingress',
+      choices=_INGRESS_MODES,
+      help='Set the ingress traffic sources allowed to call the service. For '
+      'Cloud Run (fully managed) the `--[no-]allow-unauthenticated` flag '
+      'separately controls the identities allowed to call the service.',
+      default='all')
 
 
 def AddServiceFlag(parser):
@@ -250,9 +288,11 @@ def AddNoTrafficFlag(parser):
       help='True to avoid sending traffic to the revision being deployed. '
       'Setting this flag assigns any traffic assigned to the LATEST revision '
       'to the specific revision bound to LATEST before the deployment. The '
-      'effect is that the revision being deployed will not receive traffic. '
+      'effect is that the revision being deployed will not receive traffic.\n\n'
       'After a deployment with this flag the LATEST revision will not receive '
-      'traffic on future deployments.')
+      'traffic on future deployments. To restore sending traffic to the LATEST '
+      'revision by default, run the `gcloud run services update-traffic` '
+      'command with `--to-latest`.')
 
 
 def AddDeployTagFlag(parser):
@@ -302,11 +342,11 @@ def AddUpdateTrafficFlags(parser, release_track):
     try:
       result = int(value)
     except (TypeError, ValueError):
-      raise ArgumentError('Traffic percentage value %s is not an integer.' %
-                          value)
+      raise serverless_exceptions.ArgumentError(
+          'Traffic percentage value %s is not an integer.' % value)
 
     if result < 0 or result > 100:
-      raise ArgumentError(
+      raise serverless_exceptions.ArgumentError(
           'Traffic percentage value %s is not between 0 and 100.' % value)
     return result
 
@@ -462,8 +502,12 @@ def AddMemoryFlag(parser):
 def AddCpuFlag(parser):
   parser.add_argument(
       '--cpu',
-      help='Set a CPU limit in Kubernetes cpu units. '
-      'Ex: .5, 500m, 2.')
+      help='Set a CPU limit in Kubernetes cpu units.\n\n'
+      'Cloud Run (fully managed) supports values 1, 2 and 4.'
+      '  For Cloud Run (fully managed), 4 cpus also requires a minimum 2Gi'
+      '  `--memory` value.  Examples 2, 2.0, 2000m\n\n'
+      'Cloud Run for Anthos and Knative-compatible Kubernetes clusters support'
+      '  fractional values.  Examples .5, 500m, 2')
 
 
 def _ConcurrencyValue(value):
@@ -506,15 +550,21 @@ def AddServiceAccountFlag(parser):
       'the name of a Kubernetes service account in the same namespace as the '
       'service. If not provided, the revision will use the default service '
       'account of the project, or default Kubernetes namespace service account '
-      'respectively.'.format(PLATFORM_MANAGED, PLATFORM_GKE,
-                             PLATFORM_KUBERNETES))
+      'respectively.'.format(platforms.PLATFORM_MANAGED, platforms.PLATFORM_GKE,
+                             platforms.PLATFORM_KUBERNETES))
 
 
-def AddPlatformArg(parser):
+def AddPlatformArg(parser, managed_only=False, anthos_only=False):
   """Add a platform arg."""
+  assert not (managed_only and anthos_only)
+  choices = platforms.PLATFORMS
+  if managed_only:
+    choices = platforms.PLATFORMS_MANAGED_ONLY
+  if anthos_only:
+    choices = platforms.PLATFORMS_ANTHOS_ONLY
   parser.add_argument(
       '--platform',
-      choices=_PLATFORMS,
+      choices=choices,
       action=actions.StoreProperty(properties.VALUES.run.platform),
       help='Target platform for running commands. '
       'Alternatively, set the property [run/platform]. '
@@ -560,10 +610,11 @@ def AddEgressSettingsFlag(parser):
       ' for this Service. This Service must have a VPC connector to set'
       ' VPC egress.',
       choices={
-          'private-ranges-only':
+          revision.EGRESS_SETTINGS_PRIVATE_RANGES_ONLY:
               'Default option. Sends outbound traffic to private IP addresses '
               'defined by RFC1918 through the VPC connector.',
-          'all': 'Sends all outbound traffic through the VPC connector.'
+          revision.EGRESS_SETTINGS_ALL:
+              'Sends all outbound traffic through the VPC connector.'
       })
 
 
@@ -637,11 +688,13 @@ class _ScaleValue(object):
       try:
         self.instance_count = int(value)
       except (TypeError, ValueError):
-        raise ArgumentError('Instance count value %s is not an integer '
-                            'or \'default\'.' % value)
+        raise serverless_exceptions.ArgumentError(
+            'Instance count value %s is not an integer '
+            'or \'default\'.' % value)
 
       if self.instance_count < 0:
-        raise ArgumentError('Instance count value %s is negative.' % value)
+        raise serverless_exceptions.ArgumentError(
+            'Instance count value %s is negative.' % value)
 
 
 def AddMinInstancesFlag(parser):
@@ -688,6 +741,20 @@ def AddArgsFlag(parser):
       'To reset this field to its default, pass an empty string.')
 
 
+def AddClientNameAndVersionFlags(parser):
+  """Add flags for specifying the client name and version annotations."""
+  parser.add_argument(
+      '--client-name',
+      hidden=True,
+      help="Name of the client handling the deployment. Defaults to ``global'' "
+      'if this and --client-version are both unspecified.')
+  parser.add_argument(
+      '--client-version',
+      hidden=True,
+      help='Version of the client handling the deployment. Defaults to the '
+      'current gcloud version if this and --client-name are both unspecified.')
+
+
 def _PortValue(value):
   """Returns True if port value is an int within range or 'default'."""
   try:
@@ -970,9 +1037,41 @@ def _GetTrafficChanges(args):
                                        remove_tags, clear_other_tags)
 
 
+def _GetIngressChanges(args):
+  """Returns changes to ingress traffic allowed based on the flags."""
+  platform = platforms.GetPlatform()
+  if platform == platforms.PLATFORM_MANAGED:
+    return config_changes.SetAnnotationChange(service.INGRESS_ANNOTATION,
+                                              args.ingress)
+  elif args.ingress == service.INGRESS_INTERNAL:
+    return config_changes.EndpointVisibilityChange(True)
+  elif args.ingress == service.INGRESS_ALL:
+    return config_changes.EndpointVisibilityChange(False)
+  else:
+    raise serverless_exceptions.ConfigurationError(
+        'Ingress value `{}` is not supported on platform `{}`.'.format(
+            args.ingress, platform))
+
+
 def GetConfigurationChanges(args):
   """Returns a list of changes to Configuration, based on the flags set."""
   changes = []
+
+  # Set client name and version regardless of whether or not it was specified.
+  if 'client_name' in args:
+    is_either_specified = (
+        args.IsSpecified('client_name') or args.IsSpecified('client_version'))
+    changes.append(
+        config_changes.SetClientNameAndVersionAnnotationChange(
+            args.client_name if is_either_specified else 'gcloud',
+            args.client_version
+            if is_either_specified else config.CLOUD_SDK_VERSION))
+
+  # FlagIsExplicitlySet can't be used here because args.image is also set from
+  # code in deploy.py.
+  if hasattr(args, 'image') and args.image is not None:
+    changes.append(config_changes.ImageChange(args.image))
+
   changes.extend(_GetScalingChanges(args))
   if _HasEnvChanges(args):
     changes.append(_GetEnvChanges(args))
@@ -1025,17 +1124,20 @@ def GetConfigurationChanges(args):
     changes.append(config_changes.RevisionNameChanges(args.revision_suffix))
   if 'vpc_connector' in args and args.vpc_connector:
     changes.append(config_changes.VpcConnectorChange(args.vpc_connector))
-  if 'clear_vpc_connector' in args and args.clear_vpc_connector:
-    changes.append(config_changes.ClearVpcConnectorChange())
   if FlagIsExplicitlySet(args, 'vpc_egress'):
     changes.append(
         config_changes.SetTemplateAnnotationChange(
             revision.EGRESS_SETTINGS_ANNOTATION, args.vpc_egress))
+  if 'clear_vpc_connector' in args and args.clear_vpc_connector:
+    # MUST be after 'vpc_egress' change.
+    changes.append(config_changes.ClearVpcConnectorChange())
   if 'connectivity' in args and args.connectivity:
     if args.connectivity == 'internal':
       changes.append(config_changes.EndpointVisibilityChange(True))
     elif args.connectivity == 'external':
       changes.append(config_changes.EndpointVisibilityChange(False))
+  if FlagIsExplicitlySet(args, 'ingress'):
+    changes.append(_GetIngressChanges(args))
   if 'command' in args and args.command is not None:
     # Allow passing an empty string here to reset the field
     changes.append(config_changes.ContainerCommandChange(args.command))
@@ -1061,23 +1163,12 @@ def GetService(args):
   service_re = re.compile(r'(?=^[a-z0-9-]{1,63}$)(?!^\-.*)(?!.*\-$)')
   if service_re.match(service_ref.servicesId):
     return service_ref
-  raise ArgumentError(
+  raise serverless_exceptions.ArgumentError(
       'Invalid service name [{}]. Service name must use only lowercase '
       'alphanumeric characters and dashes. Cannot begin or end with a dash, '
       'and cannot be longer than 63 characters.'.format(service_ref.servicesId))
 
 
-def GetClusterRef(cluster):
-  project = properties.VALUES.core.project.Get(required=True)
-  return resources.REGISTRY.Parse(
-      cluster.name,
-      params={
-          'projectId': project,
-          'zone': cluster.zone
-      },
-      collection='container.projects.zones.clusters')
-
-
 def PromptForRegion():
   """Prompt for region from list of available regions.
 
@@ -1160,7 +1251,7 @@ def GetAllowUnauthenticated(args, client=None, service_ref=None, prompt=False):
   return None
 
 
-def GetKubeconfig(args):
+def GetKubeconfig(file_path=None):
   """Get config from kubeconfig file.
 
   Get config from potentially 3 different places, falling back to the next
@@ -1170,7 +1261,7 @@ def GetKubeconfig(args):
   3. Default config path (~/.kube/config)
 
   Args:
-    args: Namespace, The args namespace.
+    file_path: str, the path to the kubeconfig if provided by the user
 
   Returns:
     dict: config object
@@ -1178,27 +1269,27 @@ def GetKubeconfig(args):
   Raises:
     KubeconfigError: if $KUBECONFIG is set but contains no valid paths
   """
-  if getattr(args, 'kubeconfig', None):
+  if file_path:
     return kubeconfig.Kubeconfig.LoadFromFile(
-        files.ExpandHomeDir(args.kubeconfig))
+        files.ExpandHomeDir(file_path))
   if encoding.GetEncodedValue(os.environ, 'KUBECONFIG'):
     config_paths = encoding.GetEncodedValue(os.environ,
                                             'KUBECONFIG').split(os.pathsep)
-    config = None
+    kube_config = None
     # Merge together all valid paths into single config
     for path in config_paths:
       try:
         other_config = kubeconfig.Kubeconfig.LoadFromFile(
             files.ExpandHomeDir(path))
-        if not config:
-          config = other_config
+        if not kube_config:
+          kube_config = other_config
         else:
-          config.Merge(other_config)
+          kube_config.Merge(other_config)
       except kubeconfig.Error:
         pass
-    if not config:
+    if not kube_config:
       raise KubeconfigError('No valid file paths found in $KUBECONFIG')
-    return config
+    return kube_config
   return kubeconfig.Kubeconfig.LoadFromFile(
       files.ExpandHomeDir(_DEFAULT_KUBECONFIG_PATH))
 
@@ -1210,8 +1301,15 @@ def FlagIsExplicitlySet(args, flag):
   return hasattr(args, flag) and args.IsSpecified(flag)
 
 
-def VerifyOnePlatformFlags(args, release_track, product):
-  """Raise ConfigurationError if args includes GKE only arguments."""
+def VerifyManagedFlags(args, release_track, product):
+  """Raise ConfigurationError if args aren't valid for managed Cloud Run."""
+
+  if product == Product.EVENTS and release_track != base.ReleaseTrack.ALPHA:
+    raise serverless_exceptions.ConfigurationError(
+        'The flag --platform={} is only compatible with "gcloud alpha events".'
+        ' Please provide an alternative platform value or use "gcloud alpha '
+        'events".'.format(platforms.PLATFORM_MANAGED))
+
   error_msg = ('The `{flag}` flag is not supported on the fully managed '
                'version of Cloud Run. Specify `--platform {platform}` or run '
                '`gcloud config set run/platform {platform}` to work with '
@@ -1221,99 +1319,114 @@ def VerifyOnePlatformFlags(args, release_track, product):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--connectivity=[internal|external]',
-            platform=PLATFORM_GKE,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_GKE]))
+            platform=platforms.PLATFORM_GKE,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_GKE]))
 
   if FlagIsExplicitlySet(args, 'namespace'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--namespace',
-            platform=PLATFORM_GKE,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_GKE]))
+            platform=platforms.PLATFORM_GKE,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_GKE]))
 
   if FlagIsExplicitlySet(args, 'cluster'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--cluster',
-            platform=PLATFORM_GKE,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_GKE]))
+            platform=platforms.PLATFORM_GKE,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_GKE]))
 
   if FlagIsExplicitlySet(args, 'cluster_location'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--cluster-location',
-            platform=PLATFORM_GKE,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_GKE]))
+            platform=platforms.PLATFORM_GKE,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_GKE]))
 
   if _HasSecretsChanges(args):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--[update|set|remove|clear]-secrets',
-            platform=PLATFORM_GKE,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_GKE]))
+            platform=platforms.PLATFORM_GKE,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_GKE]))
 
   if _HasConfigMapsChanges(args):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--[update|set|remove|clear]-config-maps',
-            platform=PLATFORM_GKE,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_GKE]))
+            platform=platforms.PLATFORM_GKE,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_GKE]))
 
-  if FlagIsExplicitlySet(args, 'use_http2'):
+  if FlagIsExplicitlySet(
+      args, 'use_http2') and release_track == base.ReleaseTrack.GA:
     raise serverless_exceptions.ConfigurationError(
-        error_msg.format(
-            flag='--[no-]-use-http2',
-            platform=PLATFORM_GKE,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_GKE]))
+        'The `--use-http2` flag is only supported in the beta release '
+        'track on the fully managed version of Cloud Run. Use `gcloud alpha` '
+        'to set `--use-http2` on Cloud Run (fully managed). Alternatively, '
+        'specify `--platform gke` or run '
+        '`gcloud config set run/platform gke`.')
 
   if FlagIsExplicitlySet(args, 'broker'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--broker',
-            platform=PLATFORM_GKE,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_GKE]))
+            platform=platforms.PLATFORM_GKE,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_GKE]))
 
   if FlagIsExplicitlySet(args, 'custom_type') and product == Product.EVENTS:
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--custom-type',
-            platform=PLATFORM_GKE,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_GKE]))
+            platform=platforms.PLATFORM_GKE,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_GKE]))
 
   if FlagIsExplicitlySet(args, 'kubeconfig'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--kubeconfig',
-            platform=PLATFORM_KUBERNETES,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_KUBERNETES]))
+            platform=platforms.PLATFORM_KUBERNETES,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_KUBERNETES]))
 
   if FlagIsExplicitlySet(args, 'context'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--context',
-            platform=PLATFORM_KUBERNETES,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_KUBERNETES]))
+            platform=platforms.PLATFORM_KUBERNETES,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_KUBERNETES]))
 
   if (FlagIsExplicitlySet(args, 'min_instances') and
-      release_track != base.ReleaseTrack.ALPHA):
+      release_track == base.ReleaseTrack.GA):
     raise serverless_exceptions.ConfigurationError(
-        error_msg.format(
-            flag='--min-instances',
-            platform=PLATFORM_KUBERNETES,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_KUBERNETES]))
+        'The `--min-instances` flag is not supported in the GA release track '
+        'on the fully managed version of Cloud Run. Use `gcloud beta` to set'
+        ' `--min-instances` on Cloud Run (fully managed). Alternatively, '
+        'specify `--platform kubernetes` or run `gcloud config set run/platform'
+        ' kubernetes` to work with Cloud Run for Anthos deployed on VMware.')
 
   if (FlagIsExplicitlySet(args, 'timeout') and
       release_track == base.ReleaseTrack.GA):
     if args.timeout > _FIFTEEN_MINUTES:
       raise serverless_exceptions.ConfigurationError(
-          'Timeout duration must be less than 15m.')
+          'Timeout duration must be less than 15m. Timeouts above 15m are in '
+          'Beta. Use "gcloud beta run ..." to set timeouts above 15m.')
 
   if FlagIsExplicitlySet(args, 'trigger_filters'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--trigger-filters',
-            platform=PLATFORM_GKE,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_GKE]))
+            platform=platforms.PLATFORM_GKE,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_GKE]))
 
 
 def VerifyGKEFlags(args, release_track, product):
@@ -1331,47 +1444,59 @@ def VerifyGKEFlags(args, release_track, product):
         'flag can limit which network a service is available on to reduce '
         'access.')
 
+  if (FlagIsExplicitlySet(args, 'connectivity') and
+      FlagIsExplicitlySet(args, 'ingress')):
+    raise serverless_exceptions.ConfigurationError(
+        'Cannot specify both the `--connectivity` and `--ingress` flags.'
+        ' `--connectivity` is deprecated in favor of `--ingress`.')
+
   if FlagIsExplicitlySet(args, 'region'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--region',
-            platform=PLATFORM_MANAGED,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_MANAGED]))
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
 
   if FlagIsExplicitlySet(args, 'vpc_connector'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--vpc-connector',
-            platform=PLATFORM_MANAGED,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_MANAGED]))
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
 
   if FlagIsExplicitlySet(args, 'clear_vpc_connector'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--clear-vpc-connector',
-            platform=PLATFORM_MANAGED,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_MANAGED]))
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
 
   if FlagIsExplicitlySet(args, 'vpc_egress'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--vpc-egress',
-            platform=PLATFORM_MANAGED,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_MANAGED]))
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
 
   if FlagIsExplicitlySet(args, 'kubeconfig'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--kubeconfig',
-            platform=PLATFORM_KUBERNETES,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_KUBERNETES]))
+            platform=platforms.PLATFORM_KUBERNETES,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_KUBERNETES]))
 
   if FlagIsExplicitlySet(args, 'context'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--context',
-            platform=PLATFORM_KUBERNETES,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_KUBERNETES]))
+            platform=platforms.PLATFORM_KUBERNETES,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_KUBERNETES]))
 
 
 def VerifyKubernetesFlags(args, release_track, product):
@@ -1389,125 +1514,101 @@ def VerifyKubernetesFlags(args, release_track, product):
         'flag can limit which network a service is available on to reduce '
         'access.')
 
+  if (FlagIsExplicitlySet(args, 'connectivity') and
+      FlagIsExplicitlySet(args, 'ingress')):
+    raise serverless_exceptions.ConfigurationError(
+        'Cannot specify both the `--connectivity` and `--ingress` flags.'
+        ' `--connectivity` is deprecated in favor of `--ingress`.')
+
   if FlagIsExplicitlySet(args, 'region'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--region',
-            platform=PLATFORM_MANAGED,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_MANAGED]))
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
 
   if FlagIsExplicitlySet(args, 'vpc_connector'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--vpc-connector',
             platform='managed',
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS['managed']))
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS['managed']))
 
   if FlagIsExplicitlySet(args, 'clear_vpc_connector'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--clear-vpc-connector',
-            platform=PLATFORM_MANAGED,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_MANAGED]))
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
 
   if FlagIsExplicitlySet(args, 'vpc_egress'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--vpc-egress',
-            platform=PLATFORM_MANAGED,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_MANAGED]))
+            platform=platforms.PLATFORM_MANAGED,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_MANAGED]))
 
   if FlagIsExplicitlySet(args, 'cluster'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--cluster',
-            platform=PLATFORM_GKE,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_GKE]))
+            platform=platforms.PLATFORM_GKE,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_GKE]))
 
   if FlagIsExplicitlySet(args, 'cluster_location'):
     raise serverless_exceptions.ConfigurationError(
         error_msg.format(
             flag='--cluster-location',
-            platform=PLATFORM_GKE,
-            platform_desc=_PLATFORM_SHORT_DESCRIPTIONS[PLATFORM_GKE]))
+            platform=platforms.PLATFORM_GKE,
+            platform_desc=platforms.PLATFORM_SHORT_DESCRIPTIONS[
+                platforms.PLATFORM_GKE]))
 
 
-def GetPlatform():
-  """Returns the platform to run on.
+def GetAndValidatePlatform(args, release_track, product, allow_empty=False):
+  """Returns the platform to run on and validates specified flags.
 
-  If not set by the user, this prompts the user to choose a platform and sets
-  the property so future calls to this method do continue to prompt.
-
-  Raises:
-    ArgumentError: if not platform is specified and prompting is not allowed.
-  """
-  platform = properties.VALUES.run.platform.Get()
-  if platform is None:
-    if console_io.CanPrompt():
-      platform_descs = [_PLATFORM_SHORT_DESCRIPTIONS[k] for k in _PLATFORMS]
-      index = console_io.PromptChoice(
-          platform_descs,
-          message='Please choose a target platform:',
-          cancel_option=True)
-      platform = list(_PLATFORMS.keys())[index]
-      # Set platform so we don't re-prompt on future calls to this method
-      # and so it's available to anyone who wants to know the platform.
-      properties.VALUES.run.platform.Set(platform)
-      log.status.Print(
-          'To specify the platform yourself, pass `--platform {0}`. '
-          'Or, to make this the default target platform, run '
-          '`gcloud config set run/platform {0}`.\n'.format(platform))
-    else:
-      raise ArgumentError(
-          'No platform specified. Pass the `--platform` flag or set '
-          'the [run/platform] property to specify a target platform.\n'
-          'Available platforms:\n{}'.format('\n'.join(
-              ['- {}: {}'.format(k, v) for k, v in _PLATFORMS.items()])))
-  return platform
-
-
-def GetAndValidatePlatform(args, release_track, product):
-  """Returns the platform to run on."""
-  platform = GetPlatform()
-  if platform == PLATFORM_MANAGED:
-    VerifyOnePlatformFlags(args, release_track, product)
-  elif platform == PLATFORM_GKE:
-    VerifyGKEFlags(args, release_track, product)
-  elif platform == PLATFORM_KUBERNETES:
-    VerifyKubernetesFlags(args, release_track, product)
-  if platform not in _PLATFORMS:
-    raise ArgumentError(
-        'Invalid target platform specified: [{}].\n'
-        'Available platforms:\n{}'.format(
-            platform,
-            '\n'.join(['- {}: {}'.format(k, v) for k, v in _PLATFORMS.items()
-                      ])))
-  return platform
-
-
-def ValidatePlatformIsManaged(unused_ref, unused_args, req):
-  """Validate the specified platform is managed.
-
-  This method is referenced by the declaritive iam commands which only work
-  against the managed platform.
+  A given command may support multiple platforms, but not every flag is
+  supported by every platform. This method validates that all specified flags
+  are supported by the specified platform.
 
   Args:
-    unused_ref: ref to the service.
-    unused_args: Namespace, The args namespace.
-    req: The request to be made.
+    args: Namespace, The args namespace.
+    release_track: base.ReleaseTrack, calliope release track.
+    product: Product, which product the command was executed for (e.g. Run or
+      Events).
+    allow_empty: bool, if True, allows the platform property to be unset and
+      will not prompt.
 
-  Returns:
-    Unmodified request
+  Raises:
+    ArgumentError if an unknown platform type is found.
   """
-  if GetPlatform() != PLATFORM_MANAGED:
-    raise calliope_exceptions.BadArgumentException(
-        '--platform', 'The platform [{platform}] is not supported by this '
-        'operation. Specify `--platform {managed}` or run '
-        '`gcloud config set run/platform {managed}`.'.format(
-            platform=GetPlatform(), managed=PLATFORM_MANAGED))
-  return req
+  platform = platforms.GetPlatform(prompt_if_unset=not allow_empty)
+  if platform == platforms.PLATFORM_MANAGED:
+    VerifyManagedFlags(args, release_track, product)
+  elif platform == platforms.PLATFORM_GKE:
+    VerifyGKEFlags(args, release_track, product)
+  elif platform == platforms.PLATFORM_KUBERNETES:
+    VerifyKubernetesFlags(args, release_track, product)
+  elif allow_empty and platform is None:
+    # No platform is allowed for commands that only support a single platform.
+    # It's assumed that only valid flags exist for these commands, so verifying
+    # supported flags are set is not necessary
+    return platform
+  if platform not in platforms.PLATFORMS:
+    raise serverless_exceptions.ArgumentError(
+        'Invalid target platform specified: [{}].\n'
+        'Available platforms:\n{}'.format(
+            platform, '\n'.join([
+                '- {}: {}'.format(k, v) for k, v in platforms.PLATFORMS.items()
+            ])))
+  return platform
 
 
+# TODO(b/165145546): Remove advanced build flags for 'gcloud run deploy'
 def AddBuildTimeoutFlag(parser):
   parser.add_argument(
       '--build-timeout',
@@ -1523,7 +1624,6 @@ def AddSourceFlag(parser):
   """Add deploy source flags, an image or a source for build."""
   parser.add_argument(
       '--source',
-      hidden=True,
       help='The location of the source to build. The location can be a '
       'directory on a local disk or a gzipped archive file (.tar.gz) in '
       'Google Cloud Storage. If the source is a local directory, this '
