@@ -32,11 +32,13 @@ from __future__ import unicode_literals
 
 from googlecloudsdk.api_lib.compute import utils as compute_utils
 from googlecloudsdk.api_lib.storage import storage_util
+from googlecloudsdk.api_lib.util import apis
 from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.util import completers
 
+messages = apis.GetMessagesModule('sql', 'v1beta4')
 
 _IP_ADDRESS_PART = r'(25[0-5]|2[0-4][0-9]|1?[0-9]{1,2})'  # Match decimal 0-255
 _CIDR_PREFIX_PART = r'([0-9]|[1-2][0-9]|3[0-2])'  # Match decimal 0-32
@@ -88,9 +90,7 @@ def AddInstance(parser):
 def AddInstanceArgument(parser):
   """Add the 'instance' argument to the parser."""
   parser.add_argument(
-      'instance',
-      completer=InstanceCompleter,
-      help='Cloud SQL instance ID.')
+      'instance', completer=InstanceCompleter, help='Cloud SQL instance ID.')
 
 
 # Currently, 10230 is the max storage size one can set, and 10 is the minimum.
@@ -105,36 +105,21 @@ def AddInstanceResizeLimit(parser):
 
 def AddUsername(parser):
   parser.add_argument(
-      'username',
-      completer=UserCompleter,
-      help='Cloud SQL username.')
+      'username', completer=UserCompleter, help='Cloud SQL username.')
 
 
 def AddHost(parser):
-  """Add the 'deprecated_host' arg and '--host' flag to the parser."""
-  help_text = ('Cloud SQL user\'s host name expressed as a specific IP address'
-               ' or address range. `%` denotes an unrestricted host name. '
-               'Applicable flag for MySQL instances; ignored for PostgreSQL '
-               'instances. Note, if you connect to your instance using IP '
-               'addresses, you must add your client IP address as an Authorized'
-               ' Address, even if your host name is unrestricted. For help on '
-               'how to do so, read: '
-               'https://cloud.google.com/sql/docs/mysql/configure-ip')
-
-  host = parser.add_argument_group(mutex=True)
-  # NOTICE: is_required = False is a short-term solution -- don't use elsewhere.
-  host.add_argument(
-      'deprecated_host',
-      help=help_text,
-      action=actions.DeprecationAction(
-          'deprecated_host',
-          removed=True,
-          error='Positional argument {flag_name} has been removed. Use '
-                '--host instead.'),
-      hidden=True).is_required = False
-  host.add_argument(
+  """Add the '--host' flag to the parser."""
+  parser.add_argument(
       '--host',
-      help=help_text)
+      help=('Cloud SQL user\'s host name expressed as a specific IP address'
+            ' or address range. `%` denotes an unrestricted host name. '
+            'Applicable flag for MySQL instances; ignored for all other '
+            'engines. Note, if you connect to your instance using IP '
+            'addresses, you must add your client IP address as an Authorized'
+            ' Address, even if your host name is unrestricted. For help on '
+            'how to do so, read: '
+            'https://cloud.google.com/sql/docs/mysql/configure-ip'))
 
 
 def AddAvailabilityType(parser):
@@ -153,9 +138,7 @@ def AddAvailabilityType(parser):
 
 
 def AddPassword(parser):
-  parser.add_argument(
-      '--password',
-      help='Cloud SQL user\'s password.')
+  parser.add_argument('--password', help='Cloud SQL user\'s password.')
 
 
 def AddRootPassword(parser):
@@ -175,6 +158,14 @@ def AddPromptForPassword(parser):
             'including the RETURN or ENTER key.'))
 
 
+def AddType(parser):
+  parser.add_argument('--type', help='Cloud SQL user\'s type. It determines '
+                      'the method to authenticate the user during login. '
+                      'See the list of user types at '
+                      'https://cloud.google.com/sql/docs/postgres/admin-api/'
+                      'rest/v1beta4/SqlUserType')
+
+
 # Instance create and patch flags
 
 
@@ -188,16 +179,17 @@ def AddActivationPolicy(parser):
                 'the instance should be activated and is applicable only when '
                 'the instance state is `RUNNABLE`. The default is `on-demand`. '
                 'More information on activation policies can be found here: '
-                'https://cloud.google.com/sql/faq#activation_policy')
-  ).AddToParser(parser)
+                'https://cloud.google.com/sql/faq#activation_policy'
+               )).AddToParser(parser)
 
 
 def AddAssignIp(parser):
   parser.add_argument(
       '--assign-ip',
-      help='Assign an IPv4 external address to this instance. This setting is '
-           'enabled by default when creating a new instance, but can be '
-           'disabled to use private IP connectivity.',
+      help='Assign a public IP address to the instance. This is a public, '
+      'externally available IPv4 address that you can use to connect to your '
+      'instance when properly authorized. Use --assign-ip to enable a public '
+      'IP and --no-assign-ip to disable it.',
       action=arg_parsers.StoreTrueFalseAction)
 
 
@@ -245,6 +237,41 @@ def AddBackupStartTime(parser):
             'format - HH:MM, in the UTC timezone.'))
 
 
+def AddBackupLocation(parser, allow_empty):
+  help_text = (
+      'Choose where to store your backups. Backups are stored in the closest '
+      'multi-region location to you by default. Only customize if needed.')
+  if allow_empty:
+    help_text += ' Specify empty string to revert to default.'
+  parser.add_argument('--backup-location', required=False, help=help_text)
+
+
+# Currently, MAX_BACKUP_RETENTION_COUNT=365, and MIN_BACKUP_RETENTION_COUNT=1.
+def AddRetainedBackupsCount(parser):
+  help_text = (
+      'How many backups to keep. The valid range is between 1 and 365. The '
+      'default value is 7 if not specified. Applicable only if --no-backups is '
+      'not specified.')
+  parser.add_argument(
+      '--retained-backups-count',
+      type=arg_parsers.BoundedInt(1, 365, unlimited=False),
+      help=help_text)
+
+
+# Currently, MAX_TRANSACTION_LOG_RETENTION_DAYS=7, and
+# MIN_TRANSACTION_LOG_RETENTION_DAYS=1.
+def AddRetainedTransactionLogDays(parser):
+  help_text = (
+      'How many days of transaction logs to keep. The valid range is between '
+      '1 and 7. The default value is 7 if not specified. Only valid when '
+      'point in time recovery is enabled. Keeping more days of transaction '
+      'logs requires bigger storage size')
+  parser.add_argument(
+      '--retained-transaction-log-days',
+      type=arg_parsers.BoundedInt(1, 7, unlimited=False),
+      help=help_text)
+
+
 def AddDatabaseFlags(parser, update=False):
   """Adds the `--database-flags` flag."""
   help_ = ('Comma-separated list of database flags to set on the '
@@ -267,14 +294,30 @@ def AddDatabaseFlags(parser, update=False):
 
 def AddDatabaseVersion(parser, restrict_choices=True):
   """Adds `--database-version` to the parser with choices restricted or not."""
-  choices = ['MYSQL_5_5', 'MYSQL_5_6', 'MYSQL_5_7', 'POSTGRES_9_6']
+  # Section for engine-specific content.
+  # This section is auto-generated by //cloud/storage_fe/sql/sync_engines.
+  # Do not make manual edits.
+  choices = [
+      'MYSQL_5_6',
+      'MYSQL_5_7',
+      'MYSQL_8_0',
+      'POSTGRES_9_6',
+      'POSTGRES_10',
+      'POSTGRES_11',
+      'POSTGRES_12',
+      'POSTGRES_13',
+      'SQLSERVER_2017_EXPRESS',
+      'SQLSERVER_2017_WEB',
+      'SQLSERVER_2017_STANDARD',
+      'SQLSERVER_2017_ENTERPRISE',
+  ]
+  # End of engine-specific content.
   help_text = (
       'The database engine type and version. If left unspecified, the API '
       'defaults will be used.')
   if not restrict_choices:
     help_text = ' '.join([
-        help_text,
-        'See the list of database versions at '
+        help_text, 'See the list of database versions at '
         'https://cloud.google.com/sql/docs/mysql/admin-api/v1beta4/instances'
         '#databaseVersion'
     ])
@@ -302,10 +345,14 @@ def _GetKwargsForBoolFlag(show_negated_in_help):
         'action': arg_parsers.StoreTrueFalseAction,
     }
   else:
-    return {
-        'action': 'store_true',
-        'default': None
-    }
+    return {'action': 'store_true', 'default': None}
+
+
+def AddInstanceCollation(parser):
+  parser.add_argument(
+      '--collation',
+      help='Cloud SQL server-level collation setting, which specifies '
+      'the set of rules for comparing characters in a character set.')
 
 
 def AddEnableBinLog(parser, show_negated_in_help=False):
@@ -313,9 +360,21 @@ def AddEnableBinLog(parser, show_negated_in_help=False):
   parser.add_argument(
       '--enable-bin-log',
       required=False,
-      help=(
-          'Specified if binary log should be enabled. If backup '
-          'configuration is disabled, binary log must be disabled as well.'),
+      help=('Allows for data recovery from a specific point in time, down to a '
+            'fraction of a second. Must have automatic backups enabled to use. '
+            'Make sure storage can support at least 7 days of logs.'),
+      **kwargs)
+
+
+def AddEnablePointInTimeRecovery(parser, show_negated_in_help=False):
+  kwargs = _GetKwargsForBoolFlag(show_negated_in_help)
+  parser.add_argument(
+      '--enable-point-in-time-recovery',
+      required=False,
+      help=('Allows for data recovery from a specific point in time, down to a '
+            'fraction of a second, via write-ahead logs. Must have automatic '
+            'backups enabled to use. Make sure storage can support at least 7 '
+            'days of logs.'),
       **kwargs)
 
 
@@ -420,14 +479,12 @@ def AddMaintenanceReleaseChannel(parser):
                      'their compatibility with your application prior '
                      'to the production release.'
       },
-
       help_str=("Which channel's updates to apply during the maintenance "
-                "window. If not specified, Cloud SQL chooses the timing of "
-                "updates to your instance.")).AddToParser(parser)
+                'window. If not specified, Cloud SQL chooses the timing of '
+                'updates to your instance.')).AddToParser(parser)
 
 
 def AddMaintenanceWindowDay(parser):
-  # TODO(b/79740068) Convert to ChoiceArgument when resolved.
   parser.add_argument(
       '--maintenance-window-day',
       choices=arg_parsers.DayOfWeek.DAYS,
@@ -442,6 +499,69 @@ def AddMaintenanceWindowHour(parser):
       help='Hour of day for maintenance window, in UTC time zone.')
 
 
+def AddDenyMaintenancePeriodStartDate(parser):
+  parser.add_argument(
+      '--deny-maintenance-period-start-date',
+      help='''Date when the deny maintenance period begins,
+       that is ``2020-11-01".''')
+
+
+def AddDenyMaintenancePeriodEndDate(parser):
+  parser.add_argument(
+      '--deny-maintenance-period-end-date',
+      help='Date when the deny maintenance period ends, that is ``2021-01-10".')
+
+
+def AddDenyMaintenancePeriodTime(parser):
+  parser.add_argument(
+      '--deny-maintenance-period-time',
+      help='''Time when the deny maintenance period starts or ends,
+       that is ``05:00:00".''')
+
+
+def AddInsightsConfigQueryInsightsEnabled(parser, show_negated_in_help=False):
+  kwargs = _GetKwargsForBoolFlag(show_negated_in_help)
+  parser.add_argument(
+      '--insights-config-query-insights-enabled',
+      required=False,
+      help="""Enable query insights feature to provide query and query plan
+        analytics. To disable this feature, use
+        `--no-insights-config-query-insights-enabled` instead.""",
+      **kwargs)
+
+
+def AddInsightsConfigQueryStringLength(parser):
+  parser.add_argument(
+      '--insights-config-query-string-length',
+      required=False,
+      type=arg_parsers.BoundedInt(lower_bound=256, upper_bound=4500),
+      help="""Query string length in bytes to be stored by the query insights
+        feature. Default length is 1024 bytes. Allowed range: 256 to 4500
+        bytes.""")
+
+
+def AddInsightsConfigRecordApplicationTags(parser, show_negated_in_help=False):
+  kwargs = _GetKwargsForBoolFlag(show_negated_in_help)
+  parser.add_argument(
+      '--insights-config-record-application-tags',
+      required=False,
+      help="""Allow application tags to be recorded by the query insights
+        feature. To disable application tag recording, use
+        `--no-insights-record-application-tags` flag instead.""",
+      **kwargs)
+
+
+def AddInsightsConfigRecordClientAddress(parser, show_negated_in_help=False):
+  kwargs = _GetKwargsForBoolFlag(show_negated_in_help)
+  parser.add_argument(
+      '--insights-config-record-client-address',
+      required=False,
+      help="""Allow the client address to be recorded by the query insights
+        feature. To disable client address recording, use
+        `--no-insights-config-record-client-address` instead.""",
+      **kwargs)
+
+
 def AddMemory(parser):
   parser.add_argument(
       '--memory',
@@ -513,16 +633,11 @@ def AddTier(parser, is_patch=False):
                'available here: https://cloud.google.com/sql/pricing.')
   if is_patch:
     help_text += ' WARNING: Instance will be restarted.'
-  # TODO(b/122660263): Remove when V1 instances are no longer supported.
-  # V1 deprecation notice.
-  help_text += ('\n\nIMPORTANT: First Generation instances are deprecated. '
-                'If you\'re considering any First Generation tiers, we '
-                'recommend using Second Generation instead.')
 
   parser.add_argument('--tier', '-t', required=False, help=help_text)
 
 
-def AddZone(parser, help_text):
+def AddZone(parser, help_text, enable_secondary_zone=False):
   """Adds the mutually exclusive `--gce-zone` and `--zone` to the parser."""
   zone_group = parser.add_mutually_exclusive_group()
   zone_group.add_argument(
@@ -534,10 +649,40 @@ def AddZone(parser, help_text):
           warn=('Flag `{flag_name}` is deprecated and will be removed by '
                 'release 255.0.0. Use `--zone` instead.')),
       help=help_text)
-  zone_group.add_argument(
-      '--zone',
+  if enable_secondary_zone:
+    AddZonesPrimarySecondary(zone_group, help_text)
+  else:
+    zone_group.add_argument('--zone', required=False, help=help_text)
+
+
+def AddZonesPrimarySecondary(parser, help_text):
+  """Adds the `--zone` and `--secondary-zone` to the parser."""
+
+  zone_group = parser.add_group(required=False)
+  zone_group.add_argument('--zone', required=False, help=help_text)
+  zone_group.add_argument('--secondary-zone', required=False, help=help_text)
+
+
+def AddRegion(parser):
+  parser.add_argument(
+      '--region',
       required=False,
-      help=help_text)
+      default='us-central',
+      help=('Regional location (e.g. asia-east1, us-east1). See the full '
+            'list of regions at '
+            'https://cloud.google.com/sql/docs/instance-locations.'))
+
+
+# TODO(b/31989340): add remote completion
+# TODO(b/73362371): Make specifying a location required.
+def AddLocationGroup(parser, enable_secondary_zone):
+  location_group = parser.add_mutually_exclusive_group()
+  AddRegion(location_group)
+  AddZone(
+      location_group,
+      help_text=('Preferred Compute Engine zone (e.g. us-central1-a, '
+                 'us-central1-b, etc.).'),
+      enable_secondary_zone=enable_secondary_zone)
 
 
 # Database specific flags
@@ -545,9 +690,7 @@ def AddZone(parser, help_text):
 
 def AddDatabaseName(parser):
   parser.add_argument(
-      'database',
-      completer=DatabaseCompleter,
-      help='Cloud SQL database name.')
+      'database', completer=DatabaseCompleter, help='Cloud SQL database name.')
 
 
 def AddCharset(parser):
@@ -563,7 +706,9 @@ def AddCollation(parser):
       '--collation',
       help='Cloud SQL database collation setting, which specifies '
       'the set of rules for comparing characters in a character set. Each'
-      ' database version may support a different set of collations.')
+      ' database version may support a different set of collations. For '
+      'PostgreSQL database versions, this may only be set to the collation of '
+      'the template database.')
 
 
 def AddOperationArgument(parser):
@@ -578,9 +723,19 @@ def AddOperationArgument(parser):
 
 def AddUriArgument(parser, help_text):
   """Add the 'uri' argument to the parser, with help text help_text."""
+  parser.add_argument('uri', help=help_text)
+
+
+def AddOffloadArgument(parser):
+  """Add the 'offload' argument to the parser."""
   parser.add_argument(
-      'uri',
-      help=help_text)
+      '--offload',
+      action='store_true',
+      help=(
+          'Offload an export to a temporary instance. Doing so reduces strain '
+          'on source instances and allows other operations to be performed '
+          'while the export is in progress.'
+      ))
 
 
 DEFAULT_DATABASE_IMPORT_HELP_TEXT = (
@@ -589,14 +744,19 @@ DEFAULT_DATABASE_IMPORT_HELP_TEXT = (
     'dump file includes a database statement, it will override the '
     'database set in this flag.')
 
+SQLSERVER_DATABASE_IMPORT_HELP_TEXT = (
+    'A new database into which the import is made.')
+
 
 def AddDatabase(parser, help_text, required=False):
-  """Add the '--database' flag to the parser, with help text help_text."""
-  parser.add_argument(
-      '--database',
-      '-d',
-      required=required,
-      help=help_text)
+  """Add the '--database' and '-d' flags to the parser.
+
+  Args:
+    parser: The current argparse parser to add these database flags to.
+    help_text: String, specifies the help text for the database flags.
+    required: Boolean, specifies whether the database flag is required.
+  """
+  parser.add_argument('--database', '-d', required=required, help=help_text)
 
 
 DEFAULT_DATABASE_LIST_EXPORT_HELP_TEXT = (
@@ -604,22 +764,111 @@ DEFAULT_DATABASE_LIST_EXPORT_HELP_TEXT = (
     'can be found here: https://cloud.google.com/sql/docs/mysql/admin-api/'
     'v1beta4/instances/export#exportContext.databases')
 
+SQLSERVER_DATABASE_LIST_EXPORT_HELP_TEXT = (
+    'Database from which the export is made. Information on requirements '
+    'can be found here: https://cloud.google.com/sql/docs/sqlserver/admin-api/'
+    'v1beta4/instances/export#exportContext.databases')
 
-def AddDatabaseList(parser, help_text):
-  """Add the '--database' list flag to the parser, with help text help_text."""
-  parser.add_argument(
-      '--database',
-      '-d',
-      type=arg_parsers.ArgList(min_length=1),
-      metavar='DATABASE',
-      required=False,
-      help=help_text)
+
+def AddDatabaseList(parser, help_text, required=False):
+  """Add the '--database' and '-d' list flags to the parser.
+
+  Args:
+    parser: The current argparse parser to add these database flags to.
+    help_text: String, specifies the help text for the database flags.
+    required: Boolean, specifies whether the database flag is required.
+  """
+  if required:
+    group = parser.add_group(mutex=False, required=True)
+    group.add_argument(
+        '--database',
+        '-d',
+        type=arg_parsers.ArgList(min_length=1),
+        metavar='DATABASE',
+        help=help_text)
+  else:
+    parser.add_argument(
+        '--database',
+        '-d',
+        type=arg_parsers.ArgList(min_length=1),
+        metavar='DATABASE',
+        required=False,
+        help=help_text)
 
 
 def AddUser(parser, help_text):
   """Add the '--user' flag to the parser, with help text help_text."""
   parser.add_argument('--user', help=help_text)
 
+
+def AddEncryptedBakFlags(parser):
+  """Add the flags for importing encrypted BAK files.
+
+  Add the --cert-path, --pvk-path, --pvk-password and
+  --prompt-for-pvk-password flags to the parser
+
+  Args:
+    parser: The current argparse parser to add these database flags to.
+  """
+  enc_group = parser.add_group(
+      mutex=False,
+      required=False,
+      help='Encryption info to support importing an encrypted .bak file')
+  enc_group.add_argument(
+      '--cert-path',
+      required=True,
+      help=('Path to the encryption certificate file in Google Cloud Storage '
+            'associated with the BAK file. The URI is in the form '
+            '`gs://bucketName/fileName`.'))
+  enc_group.add_argument(
+      '--pvk-path',
+      required=True,
+      help=('Path to the encryption private key file in Google Cloud Storage '
+            'associated with the BAK file. The URI is in the form '
+            '`gs://bucketName/fileName`.'))
+  password_group = enc_group.add_group(mutex=True, required=True)
+  password_group.add_argument(
+      '--pvk-password',
+      help='The private key password associated with the BAK file.')
+  password_group.add_argument(
+      '--prompt-for-pvk-password',
+      action='store_true',
+      help=(
+          'Prompt for the private key password associated with the BAK file '
+          'with character echo disabled. The password is all typed characters '
+          'up to but not including the RETURN or ENTER key.'))
+
+
+def AddRescheduleType(parser):
+  """Add the flag to specify reschedule type.
+
+  Args:
+    parser: The current argparse parser to add this to.
+  """
+  choices = [
+      messages.Reschedule.RescheduleTypeValueValuesEnum.IMMEDIATE.name,
+      messages.Reschedule.RescheduleTypeValueValuesEnum.NEXT_AVAILABLE_WINDOW
+      .name,
+      messages.Reschedule.RescheduleTypeValueValuesEnum.SPECIFIC_TIME.name,
+  ]
+  help_text = 'The type of reschedule operation to perform.'
+  parser.add_argument(
+      '--reschedule-type', choices=choices, required=True, help=help_text)
+
+
+def AddScheduleTime(parser):
+  """Add the flag for maintenance reschedule schedule time.
+
+  Args:
+    parser: The current argparse parser to add this to.
+  """
+  parser.add_argument(
+      '--schedule-time',
+      type=arg_parsers.Datetime.Parse,
+      help=('When specifying SPECIFIC_TIME, the date and time at which to '
+            'schedule the maintenance in ISO 8601 format.'))
+
+
 INSTANCES_USERLABELS_FORMAT = ':(settings.userLabels:alias=labels:label=LABELS)'
 
 INSTANCES_FORMAT_COLUMNS = [
@@ -628,8 +877,7 @@ INSTANCES_FORMAT_COLUMNS = [
     'ip_addresses.filter("type:PRIMARY").*extract(ip_address).flatten()'
     '.yesno(no="-"):label=PRIMARY_ADDRESS',
     'ip_addresses.filter("type:PRIVATE").*extract(ip_address).flatten()'
-    '.yesno(no="-"):label=PRIVATE_ADDRESS',
-    'state:label=STATUS'
+    '.yesno(no="-"):label=PRIVATE_ADDRESS', 'state:label=STATUS'
 ]
 
 
@@ -686,9 +934,28 @@ TIERS_FORMAT = """
   )
 """
 
-USERS_FORMAT_BETA = """
+USERS_FORMAT = """
   table(
     name.yesno(no='(anonymous)'),
-    host
+    host,
+    type.yesno(no='BUILT_IN')
   )
 """
+
+USERS_FORMAT_BETA = USERS_FORMAT
+
+
+USERS_FORMAT_ALPHA = USERS_FORMAT_BETA
+
+
+def AddActiveDirectoryDomain(parser):
+  """Adds the '--active-directory-domain' flag to the parser.
+
+  Args:
+    parser: The current argparse parser to add this to.
+  """
+  help_text = (
+      'Managed Service for Microsoft Active Directory domain this instance is '
+      'joined to. Only available for SQL Server instances.'
+  )
+  parser.add_argument('--active-directory-domain', help=help_text)
