@@ -19,12 +19,13 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+import enum
 import errno
 import getpass
 import os
 import re
 import string
-import enum
+import tempfile
 
 from googlecloudsdk.api_lib.oslogin import client as oslogin_client
 from googlecloudsdk.command_lib.oslogin import oslogin_utils
@@ -269,7 +270,7 @@ class Keys(object):
       # which are relatively inconsequential.
       decoded_key = key_string.strip()
       if isinstance(key_string, six.binary_type):
-        decoded_key = decoded_key.decode('utf8', 'replace')
+        decoded_key = decoded_key.decode('utf-8', 'replace')
       parts = decoded_key.split(' ', 2)
       if len(parts) < 2:
         raise InvalidKeyError('Public key [{}] is invalid.'.format(key_string))
@@ -524,6 +525,27 @@ class Keys(object):
       cmd = KeygenCommand(self.key_file, allow_passphrase=allow_passphrase)
       cmd.Run(self.env)
 
+    if self.env.suite is Suite.PUTTY:
+      # This is to fix an encoding issue with PPK's we generated that was
+      # ignored in versions of PuTTY <=0.70, but became invalid in version 0.71.
+      # Since this only affects the PPK, we don't need to generate a new key; we
+      # can just correct the encoding of the PPK if necessary. We use a sentinel
+      # file in the config dir to check if the encoding is already correct.
+      valid_ppk_sentinel = config.Paths().valid_ppk_sentinel_file
+      if not os.path.exists(valid_ppk_sentinel):
+        if key_files_validity is KeyFileStatus.PRESENT:  # Initial validity
+          cmd = KeygenCommand(
+              self.key_file, allow_passphrase=False, reencode_ppk=True)
+          cmd.Run(self.env)
+        try:
+          files.WriteFileContents(valid_ppk_sentinel, '')
+        except files.Error as e:
+          # It's possible that writing the sentinel file fails, which means
+          # we'll potentially have to re-encode the PPK again the next time an
+          # SSH/SCP command is run. But we shouldn't let this prevent the user
+          # from running their current command.
+          log.debug('Failed to create sentinel file: [{}]'.format(e))
+
 
 class KnownHosts(object):
   """Represents known hosts file, supports read, write and basic key management.
@@ -701,7 +723,8 @@ def _MetadataHasOsloginEnable(metadata):
 
 
 def CheckForOsloginAndGetUser(instance, project, requested_user, public_key,
-                              expiration_time, release_track):
+                              expiration_time, release_track,
+                              username_requested=False):
   """Check instance/project metadata for oslogin and return updated username.
 
   Check to see if OS Login is enabled in metadata and if it is, return
@@ -718,6 +741,8 @@ def CheckForOsloginAndGetUser(instance, project, requested_user, public_key,
       not be set to expire.  If not None, an existing key may be modified
       with the new expiry.
     release_track: release_track, The object representing the release track.
+    username_requested: bool, True if the user has passed a specific username in
+      the args.
 
   Returns:
     tuple, A string containing the oslogin username and a boolean indicating
@@ -742,15 +767,17 @@ def CheckForOsloginAndGetUser(instance, project, requested_user, public_key,
         'OS Login is enabled on Instance/Project, but is not available '
         'in the {0} version of gcloud.'.format(release_track.id))
     return requested_user, use_oslogin
-  user_email = properties.VALUES.core.account.Get()
+  user_email = (properties.VALUES.auth.impersonate_service_account.Get()
+                or properties.VALUES.core.account.Get())
 
-  # Check to see if public key is already in profile, import if not, and update
-  # the expiration if it is.
+  # Check to see if public key is already in profile and POSIX information
+  # exists associated with the project. If either are not set, import an SSH
+  # public key. Otherwise update the expiration time if needed.
   login_profile = oslogin.GetLoginProfile(user_email, project.name)
   keys = oslogin_utils.GetKeyDictionaryFromProfile(
       user_email, oslogin, profile=login_profile)
   fingerprint = oslogin_utils.FindKeyInKeyList(public_key, keys)
-  if not fingerprint:
+  if not fingerprint or not login_profile.posixAccounts:
     import_response = oslogin.ImportSshPublicKey(user_email, public_key,
                                                  expiration_time)
     login_profile = import_response.loginProfile
@@ -771,8 +798,15 @@ def CheckForOsloginAndGetUser(instance, project, requested_user, public_key,
     elif pa.primary:
       oslogin_user = pa.username
 
-  log.out.Print('Using OS Login user [{0}] instead of default user [{1}]'
-                .format(oslogin_user, requested_user))
+  # If the user passed in a specific username to the command, show a message
+  # to the user, otherwise just add a message to the log.
+  if username_requested:
+    log.status.Print(
+        'Using OS Login user [{0}] instead of requested user [{1}]'
+        .format(oslogin_user, requested_user))
+  else:
+    log.info('Using OS Login user [{0}] instead of default user [{1}]'.format(
+        oslogin_user, requested_user))
   return oslogin_user, use_oslogin
 
 
@@ -1029,12 +1063,16 @@ class KeygenCommand(object):
       - Running in an OpenSSH environment (Linux and Mac)
       - Running in interactive mode (from an actual TTY)
       - Prompts are enabled in gcloud
+    reencode_ppk: bool, If True, reencode the PPK file if it was generated with
+      a bad encoding, instead of generating a new key. This is only valid for
+      PuTTY.
   """
 
-  def __init__(self, identity_file, allow_passphrase=True):
+  def __init__(self, identity_file, allow_passphrase=True, reencode_ppk=False):
     """Construct a suite independent `ssh-keygen` command."""
     self.identity_file = identity_file
     self.allow_passphrase = allow_passphrase
+    self.reencode_ppk = reencode_ppk
 
   def Build(self, env=None):
     """Construct the actual command according to the given environment.
@@ -1059,6 +1097,8 @@ class KeygenCommand(object):
         args.extend(['-N', ''])  # Empty passphrase
       args.extend(['-t', 'rsa', '-f', self.identity_file])
     else:
+      if self.reencode_ppk:
+        args.append('--reencode-ppk')
       args.append(self.identity_file)
 
     return args
@@ -1138,6 +1178,7 @@ class SSHCommand(object):
     self.tty = tty
     self.iap_tunnel_args = iap_tunnel_args
     self.remainder = remainder
+    self._remote_command_file = None
 
   def Build(self, env=None):
     """Construct the actual command according to the given environment.
@@ -1183,9 +1224,19 @@ class SSHCommand(object):
       args.extend(self.remainder)
 
     if self.remote_command:
-      if env.suite is Suite.OPENSSH:  # Putty doesn't like double dash
+      if env.suite is Suite.OPENSSH:
         args.append('--')
-      args.extend(self.remote_command)
+        args.extend(self.remote_command)
+      elif tty:
+        # Unlike plink.exe, putty.exe doesn't support passing a remote command
+        # with spaces on the command line; however the -m flag lets one pass in
+        # a local file from which to read the remote command.
+        with tempfile.NamedTemporaryFile(
+            mode='w+t', delete=False) as self._remote_command_file:
+          self._remote_command_file.write(' '.join(self.remote_command))
+        args.extend(['-m', self._remote_command_file.name])
+      else:
+        args.extend(self.remote_command)
     return args
 
   def Run(self, env=None, force_connect=False,
@@ -1224,6 +1275,19 @@ class SSHCommand(object):
 
     status = execution_utils.Exec(args, no_exit=True, in_str=in_str,
                                   **extra_popen_kwargs)
+
+    # This should only happen if we're using putty.exe with a remote command. In
+    # that case the file created earlier via NamedTemporaryFile has to be
+    # deleted manually due to https://bugs.python.org/issue14243 on Windows.
+    if self._remote_command_file:
+      try:
+        os.remove(self._remote_command_file.name)
+      except OSError:
+        # Not worth crashing over.
+        log.debug('Failed to delete remote command file [{}]'.format(
+            self._remote_command_file.name))
+        pass
+
     if status == env.ssh_exit_code:
       raise CommandError(args[0], return_code=status)
     return status
