@@ -39,37 +39,11 @@ from googlecloudsdk.core import resources
 from googlecloudsdk.core.console import progress_tracker
 import six
 
-_WORKFLOW_DIR = '../workflows/image_import/'
-_IMPORT_WORKFLOW = _WORKFLOW_DIR + 'import_image.wf.json'
-_IMPORT_FROM_IMAGE_WORKFLOW = _WORKFLOW_DIR + 'import_from_image.wf.json'
-_IMPORT_AND_TRANSLATE_WORKFLOW = _WORKFLOW_DIR + 'import_and_translate.wf.json'
 _WORKFLOWS_URL = ('https://github.com/GoogleCloudPlatform/compute-image-tools/'
                   'tree/master/daisy_workflows/image_import')
 _OUTPUT_FILTER = ['[Daisy', '[import-', 'starting build', '  import', 'ERROR']
 
 
-def _IsLocalFile(file_name):
-  return not (file_name.startswith('gs://') or
-              file_name.startswith('https://'))
-
-
-def _UploadToGcsGsutil(local_path, dest_path):
-  """Uploads a local file to GCS using gsutil."""
-  retcode = storage_util.RunGsutilCommand('cp', [local_path, dest_path])
-  if retcode != 0:
-    log.err.Print('Failed to upload file. See {} for details.'.format(
-        log.GetLogFilePath()))
-    raise exceptions.FailedSubCommand(
-        ['gsutil', 'cp', local_path, dest_path], retcode)
-  return dest_path
-
-
-def _GetTranslateWorkflow(args):
-  if args.os:
-    return os_choices.OS_CHOICES_MAP[args.os]
-  return args.custom_workflow
-
-
 def _AppendTranslateWorkflowArg(args, import_args):
   if args.os:
     daisy_utils.AppendArg(import_args, 'os', args.os)
@@ -99,7 +73,6 @@ def _CheckImageName(image_name):
 
 def _CheckForExistingImage(image_name, compute_holder):
   """Check that the destination image does not already exist."""
-  _CheckImageName(image_name)
   image_ref = resources.REGISTRY.Parse(
       image_name,
       collection='compute.images',
@@ -120,9 +93,10 @@ def _CheckForExistingImage(image_name, compute_holder):
 
 @base.ReleaseTracks(base.ReleaseTrack.GA)
 class Import(base.CreateCommand):
-  """Import an image into Google Compute Engine."""
+  """Import an image into Compute Engine."""
 
   _OS_CHOICES = os_choices.OS_CHOICES_IMAGE_IMPORT_GA
+  _OS_FLAG_REQUIRED = True
 
   def __init__(self, *args, **kwargs):
     self.storage_client = storage_api.StorageClient()
@@ -140,17 +114,18 @@ class Import(base.CreateCommand):
     source = parser.add_mutually_exclusive_group(required=True)
     source.add_argument(
         '--source-file',
-        help=("""A local file, or the Google Cloud Storage URI of the virtual
+        help=("""A local file, or the Cloud Storage URI of the virtual
               disk file to import. For example: ``gs://my-bucket/my-image.vmdk''
               or ``./my-local-image.vmdk''"""),
     )
     flags.SOURCE_IMAGE_ARG.AddArgument(source, operation_type='import')
 
-    workflow = parser.add_mutually_exclusive_group(required=True)
+    workflow = parser.add_mutually_exclusive_group(
+        required=cls._OS_FLAG_REQUIRED)
     workflow.add_argument(
         '--os',
         choices=sorted(cls._OS_CHOICES),
-        help='Specifies the OS of the image being imported.'
+        help='Specifies the OS of the disk image being imported.'
     )
     workflow.add_argument(
         '--data-disk',
@@ -162,11 +137,10 @@ class Import(base.CreateCommand):
     workflow.add_argument(
         '--custom-workflow',
         help=("""\
-              Specifies a custom workflow to use for image translation.
-              Workflow should be relative to the image_import directory here:
-              []({0}). For example: ``{1}''""".format(
-                  _WORKFLOWS_URL,
-                  os_choices.OS_CHOICES_MAP[sorted(cls._OS_CHOICES)[0]])),
+              Specifies a custom workflow to use for image translation. Workflow
+              should be relative to the image_import directory here: []({0}).
+              For example: `debian/translate_debian_9.wf.json'""".format(
+                  _WORKFLOWS_URL)),
         hidden=True
     )
 
@@ -176,12 +150,14 @@ class Import(base.CreateCommand):
         '--guest-environment',
         action='store_true',
         default=True,
-        help='Google Guest Environment will be installed on the image.')
+        help='Installs the guest environment on the image.'
+             ' See '
+             'https://cloud.google.com/compute/docs/images/guest-environment.')
 
     parser.add_argument(
         '--network',
         help=('Name of the network in your project to use for the image import.'
-              ' The network must have access to Google Cloud Storage. If not '
+              ' The network must have access to Cloud Storage. If not '
               'specified, the network named `default` is used.'),
     )
 
@@ -195,67 +171,81 @@ class Import(base.CreateCommand):
               'Region or zone should be specified if this field is specified.'),
     )
 
+    parser.add_argument(
+        '--family',
+        help='Family to set for the imported image.')
+
+    parser.add_argument(
+        '--description',
+        help='Description to set for the imported image.')
+
     parser.display_info.AddCacheUpdater(flags.ImagesCompleter)
 
+    parser.add_argument(
+        '--storage-location',
+        help="""\
+      Specifies a Cloud Storage location, either regional or multi-regional,
+      where image content is to be stored. If not specified, the multi-region
+      location closest to the source is chosen automatically.
+      """)
+
+    parser.add_argument(
+        '--sysprep-windows',
+        action='store_true',
+        hidden=True,
+        help='Whether to generalize the image using Windows Sysprep.')
+
+    daisy_utils.AddNoAddressArg(
+        parser,
+        'image import',
+        'https://cloud.google.com/compute/docs/import/importing-virtual-disks#no-external-ip'
+    )
+
   def Run(self, args):
     compute_holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
+
     # Fail early if the requested image name is invalid or already exists.
     _CheckImageName(args.image_name)
     _CheckForExistingImage(args.image_name, compute_holder)
 
-    import_stager = self._CreateImportStager(args)
-    import_metadata = self._Stage(import_stager)
+    stager = self._CreateImportStager(args)
+    import_metadata = stager.Stage()
 
     # TODO(b/79591894): Once we've cleaned up the Argo output, replace this
     # warning message with a ProgressTracker spinner.
     log.warning('Importing image. This may take up to 2 hours.')
+
     tags = ['gce-daisy-image-import']
 
-    return self._RunImageImport(args, import_stager, import_metadata, tags)
+    return self._RunImageImport(args, import_metadata, tags, _OUTPUT_FILTER)
 
-  def _Stage(self, import_stager):
-    """Prepares for import.
-
-    Args:
-      import_stager: BaseImportStager to do the actual staging job.
-
-    Returns:
-      list of str, which contains metadata for the import step
-    """
-    return import_stager.StageForDaisy()
-
-  def _RunImageImport(self, args, import_stager, import_metadata, tags):
-    """Run actual image import.
-
-    Args:
-      args: list of str, CLI args that might contain network/subnet args.
-      import_stager: BaseImportStager, to do actual stage steps.
-      import_metadata: list of str, contains metadata used by import. It can be
-        daisy vars or import wrapper args.
-      tags: A list of strings for adding tags to the Argo build.
-
-    Returns:
-      A cloud build that executes importing.
-    """
-    return daisy_utils.RunDaisyBuild(
+  def _RunImageImport(self, args, import_args, tags, output_filter):
+    return daisy_utils.RunImageImport(
         args,
-        import_stager.GetDaisyWorkflow(),
-        ','.join(import_metadata),
-        tags=tags,
-        daisy_bucket=import_stager.GetDaisyBucket(),
-        user_zone=properties.VALUES.compute.zone.Get(),
-        output_filter=_OUTPUT_FILTER)
-
-  def _MakeGcsUri(self, uri):
-    return daisy_utils.MakeGcsUri(uri)
+        import_args,
+        tags,
+        _OUTPUT_FILTER,
+        release_track=self.ReleaseTrack().id.lower()
+        if self.ReleaseTrack() else None)
 
   def _CreateImportStager(self, args):
     if args.source_image:
-      return ImportFromImageStager(self.storage_client, args)
-    elif _IsLocalFile(args.source_file):
-      return ImportFromLocalFileStager(self.storage_client, args)
+      return ImportFromImageStager(
+          self.storage_client, args)
+
+    if daisy_utils.IsLocalFile(args.source_file):
+      return ImportFromLocalFileStager(
+          self.storage_client, args)
+
+    try:
+      gcs_uri = daisy_utils.MakeGcsObjectUri(args.source_file)
+    except storage_util.InvalidObjectNameError:
+      raise exceptions.InvalidArgumentException(
+          'source-file',
+          'must be a path to an object in Google Cloud Storage')
     else:
-      return ImportFromGSFileStager(self.storage_client, args, self._MakeGcsUri)
+      return ImportFromGSFileStager(
+          self.storage_client, args, gcs_uri)
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -270,66 +260,25 @@ class BaseImportStager(object):
   def __init__(self, storage_client, args):
     self.storage_client = storage_client
     self.args = args
-
-    self._CreateDaisyBucket()
-
-  def _CreateDaisyBucket(self):
-    # Create Daisy bucket in default GS location (US Multi-regional)
-    # This is default behaviour for all types of import except from a file in GS
     self.daisy_bucket = self.GetAndCreateDaisyBucket()
 
-  def GetDaisyBucket(self):
-    """Returns the name of Daisy scratch bucket.
-
-    Returns:
-      A string. Name of the Daisy scratch bucket used for running import.
-    """
-    return self.daisy_bucket
-
-  def StageForDaisy(self):
-    """Prepares import for execution and returns daisy variables.
-
-    It supports native daisy, which is on the path of deprecation for image
-    import feature.
-
-    Returns:
-      daisy_vars - array of strings, Daisy variables.
-    """
-    daisy_vars = []
-    self._BuildDaisyVars(daisy_vars)
-    return daisy_vars
-
-  def StageForImporter(self):
-    """Prepares import for execution and returns import variables.
+  def Stage(self):
+    """Prepares for import args.
 
     It supports running new import wrapper (gce_vm_image_import).
 
     Returns:
-      import_args - array of strings, import variables.
+      import_args - array of strings, import args.
     """
     import_args = []
-    self._BuildImportArgs(import_args)
-    return import_args
 
-  def _BuildDaisyVars(self, daisy_vars):
-    daisy_vars.append('image_name={}'.format(self.args.image_name))
-
-    if not self.args.guest_environment:
-      daisy_vars.append('install_gce_packages={}'.format('false'))
-
-    daisy_vars.extend(daisy_utils.ExtractNetworkAndSubnetDaisyVariables(
-        self.args, daisy_utils.ImageOperation.IMPORT))
-
-  def _BuildImportArgs(self, import_args):
-    """Build args to support running new import wrapper - gce_vm_image_import.
-
-    Args:
-      import_args: array of str, args to build.
-    """
     daisy_utils.AppendArg(import_args, 'zone',
                           properties.VALUES.compute.zone.Get())
+    if self.args.storage_location:
+      daisy_utils.AppendArg(import_args, 'storage_location',
+                            self.args.storage_location)
     daisy_utils.AppendArg(import_args, 'scratch_bucket_gcs_path',
-                          'gs://{0}/'.format(self.GetDaisyBucket()))
+                          'gs://{0}/'.format(self.daisy_bucket))
     daisy_utils.AppendArg(import_args, 'timeout',
                           '{}s'.format(daisy_utils.GetDaisyTimeout(self.args)))
 
@@ -338,33 +287,39 @@ class BaseImportStager(object):
     daisy_utils.AppendBoolArg(import_args, 'no_guest_environment',
                               not self.args.guest_environment)
     daisy_utils.AppendNetworkAndSubnetArgs(self.args, import_args)
+    daisy_utils.AppendArg(import_args, 'description', self.args.description)
+    daisy_utils.AppendArg(import_args, 'family', self.args.family)
+    if 'sysprep_windows' in self.args:
+      daisy_utils.AppendBoolArg(import_args, 'sysprep_windows',
+                                self.args.sysprep_windows)
+    if 'no_address' in self.args:
+      daisy_utils.AppendBoolArg(import_args, 'no_external_ip',
+                                self.args.no_address)
 
-  @abc.abstractmethod
-  def GetDaisyWorkflow(self):
-    raise NotImplementedError
+    return import_args
 
   def GetAndCreateDaisyBucket(self):
-    bucket_name = daisy_utils.GetDaisyBucketName()
-    self.storage_client.CreateBucketIfNotExists(bucket_name)
-    return bucket_name
+    return daisy_utils.CreateDaisyBucketInProject(self.GetBucketLocation(),
+                                                  self.storage_client)
+
+  def GetBucketLocation(self):
+    if self.args.storage_location:
+      return self.args.storage_location
+
+    return None
 
 
 class ImportFromImageStager(BaseImportStager):
   """Image import stager from an existing image."""
 
-  def _BuildDaisyVars(self, daisy_vars):
-    super(ImportFromImageStager, self)._BuildDaisyVars(daisy_vars)
-    daisy_vars.append(
-        'translate_workflow={}'.format(_GetTranslateWorkflow(self.args)))
+  def Stage(self):
+    import_args = []
 
-    source_name = self._GetSourceImage()
-    daisy_vars.append('source_image={}'.format(source_name))
-
-  def _BuildImportArgs(self, import_args):
-    source_image_name = self._GetSourceImage()
-    daisy_utils.AppendArg(import_args, 'source_image', source_image_name)
+    daisy_utils.AppendArg(import_args, 'source_image', self.args.source_image)
     _AppendTranslateWorkflowArg(self.args, import_args)
-    super(ImportFromImageStager, self)._BuildImportArgs(import_args)
+
+    import_args.extend(super(ImportFromImageStager, self).Stage())
+    return import_args
 
   def _GetSourceImage(self):
     ref = resources.REGISTRY.Parse(
@@ -374,22 +329,15 @@ class ImportFromImageStager(BaseImportStager):
     source_name = ref.RelativeName()[len(ref.Parent().RelativeName() + '/'):]
     return source_name
 
-  def GetDaisyWorkflow(self):
-    return _IMPORT_FROM_IMAGE_WORKFLOW
-
 
 class BaseImportFromFileStager(BaseImportStager):
   """Abstract image import stager for import from a file."""
 
-  def _BuildDaisyVars(self, daisy_vars):
-    super(BaseImportFromFileStager, self)._BuildDaisyVars(daisy_vars)
-    # Import and (maybe) translate from the scratch bucket.
-    daisy_vars.append('source_disk_file={}'.format(self.gcs_uri))
-    if not self.args.data_disk:
-      daisy_vars.append(
-          'translate_workflow={}'.format(_GetTranslateWorkflow(self.args)))
+  def Stage(self):
+    self._FileStage()
+
+    import_args = []
 
-  def _BuildImportArgs(self, import_args):
     # Import and (maybe) translate from the scratch bucket.
     daisy_utils.AppendArg(import_args, 'source_file', self.gcs_uri)
     if self.args.data_disk:
@@ -397,35 +345,24 @@ class BaseImportFromFileStager(BaseImportStager):
     else:
       _AppendTranslateWorkflowArg(self.args, import_args)
 
-    super(BaseImportFromFileStager, self)._BuildImportArgs(import_args)
-
-  def GetDaisyWorkflow(self):
-    if self.args.data_disk:
-      return _IMPORT_WORKFLOW
-    else:
-      return _IMPORT_AND_TRANSLATE_WORKFLOW
-
-  def StageForDaisy(self):
-    self._FileStage()
-    return super(BaseImportFromFileStager, self).StageForDaisy()
-
-  def StageForImporter(self):
-    self._FileStage()
-    return super(BaseImportFromFileStager, self).StageForImporter()
+    import_args.extend(super(BaseImportFromFileStager, self).Stage())
+    return import_args
 
   def _FileStage(self):
     """Prepare image file for importing."""
     # If the file is an OVA file, print a warning.
-    if self.args.source_file.endswith('.ova'):
+    if self.args.source_file.lower().endswith('.ova'):
       log.warning(
           'The specified input file may contain more than one virtual disk. '
-          'Only the first vmdk disk will be imported.')
-    elif (self.args.source_file.endswith('.tar.gz')
-          or self.args.source_file.endswith('.tgz')):
+          'Only the first vmdk disk will be imported. To import a .ova'
+          'completely, please try \'gcloud compute instances import\''
+          'instead.')
+    elif (self.args.source_file.lower().endswith('.tar.gz')
+          or self.args.source_file.lower().endswith('.tgz')):
       raise exceptions.BadFileException(
           '`gcloud compute images import` does not support compressed '
           'archives. Please extract your image and try again.\n If you got '
-          'this file by exporting an image from Compute Engine (e.g. by '
+          'this file by exporting an image from Compute Engine (e.g., by '
           'using `gcloud compute images export`) then you can instead use '
           '`gcloud compute images create` to create your image from your '
           '.tar.gz file.')
@@ -441,7 +378,8 @@ class ImportFromLocalFileStager(BaseImportFromFileStager):
 
   def _CopySourceFileToScratchBucket(self):
     return self._UploadToGcs(
-        self.args.async, self.args.source_file, self.daisy_bucket, uuid.uuid4())
+        self.args.async_, self.args.source_file, self.daisy_bucket,
+        uuid.uuid4())
 
   def _UploadToGcs(self, is_async, local_path, daisy_bucket, image_uuid):
     """Uploads a local file to GCS. Returns the gs:// URI to that file."""
@@ -449,39 +387,30 @@ class ImportFromLocalFileStager(BaseImportFromFileStager):
     dest_path = 'gs://{0}/tmpimage/{1}-{2}'.format(
         daisy_bucket, image_uuid, file_name)
     if is_async:
-      log.status.Print('Async: Once upload is complete, your image will be '
+      log.status.Print('Async: After upload is complete, your image will be '
                        'imported from Cloud Storage asynchronously.')
     with progress_tracker.ProgressTracker(
         'Copying [{0}] to [{1}]'.format(local_path, dest_path)):
-      # TODO(b/109938541): Remove gsutil implementation after the new
-      # implementation seems stable.
-      use_gsutil = properties.VALUES.storage.use_gsutil.GetBool()
-      if use_gsutil:
-        return _UploadToGcsGsutil(local_path, dest_path)
-      else:
-        return self._UploadToGcsStorageApi(local_path, dest_path)
+      return self._UploadToGcsStorageApi(local_path, dest_path)
 
   def _UploadToGcsStorageApi(self, local_path, dest_path):
-    """Uploads a local file to GCS using the gcloud storage api client."""
+    """Uploads a local file to Cloud Storage using the gcloud storage api client."""
     dest_object = storage_util.ObjectReference.FromUrl(dest_path)
     self.storage_client.CopyFileToGCS(local_path, dest_object)
     return dest_path
 
 
 class ImportFromGSFileStager(BaseImportFromFileStager):
-  """Image import stager from a file in GCS."""
+  """Image import stager from a file in Cloud Storage."""
 
-  def __init__(self, storage_client, args, makeGcsUri):
-    self.source_file_gcs_uri = makeGcsUri(args.source_file)
-    super(ImportFromGSFileStager, self).__init__(storage_client, args)
+  def __init__(self, storage_client, args, gcs_uri):
+    self.source_file_gcs_uri = gcs_uri
+    super(ImportFromGSFileStager, self).__init__(
+        storage_client, args)
 
-  def GetAndCreateDaisyBucket(self):
-    bucket_location = self.storage_client.GetBucketLocationForFile(
+  def GetBucketLocation(self):
+    return self.storage_client.GetBucketLocationForFile(
         self.source_file_gcs_uri)
-    bucket_name = daisy_utils.GetDaisyBucketName(bucket_location)
-    self.storage_client.CreateBucketIfNotExists(
-        bucket_name, location=bucket_location)
-    return bucket_name
 
   def _CopySourceFileToScratchBucket(self):
     image_file = os.path.basename(self.source_file_gcs_uri)
@@ -499,53 +428,101 @@ class ImportFromGSFileStager(BaseImportFromFileStager):
 
 @base.ReleaseTracks(base.ReleaseTrack.BETA)
 class ImportBeta(Import):
-  """Import an image into Google Compute Engine for Beta releases."""
+  """Import an image into Compute Engine for beta releases."""
 
   _OS_CHOICES = os_choices.OS_CHOICES_IMAGE_IMPORT_BETA
+  _OS_FLAG_REQUIRED = False
 
-  def _MakeGcsUri(self, uri):
-    try:
-      return daisy_utils.MakeGcsObjectOrPathUri(uri)
-    except storage_util.InvalidObjectNameError:
-      raise exceptions.InvalidArgumentException(
-          'source-file', 'must be a path to an object in Google Cloud Storage')
+  @classmethod
+  def Args(cls, parser):
+    super(ImportBeta, cls).Args(parser)
+    daisy_utils.AddExtraCommonDaisyArgs(parser)
 
-  def _Stage(self, import_stager):
-    return import_stager.StageForImporter()
-
-  def _RunImageImport(self, args, import_stager, import_metadata, tags):
-    return daisy_utils.RunImageImport(args, import_metadata, tags,
-                                      _OUTPUT_FILTER)
+  def _RunImageImport(self, args, import_args, tags, output_filter):
+    return daisy_utils.RunImageImport(
+        args,
+        import_args,
+        tags,
+        _OUTPUT_FILTER,
+        release_track=self.ReleaseTrack().id.lower()
+        if self.ReleaseTrack() else None,
+        docker_image_tag=args.docker_image_tag)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
 class ImportAlpha(ImportBeta):
-  """Import an image into Google Compute Engine for Alpha releases."""
+  """Import an image into Compute Engine for alpha releases."""
 
   _OS_CHOICES = os_choices.OS_CHOICES_IMAGE_IMPORT_ALPHA
 
 
 Import.detailed_help = {
-    'brief': 'Import an image into Google Compute Engine',
-    'DESCRIPTION': """\
+    'brief': 'Import an image into Compute Engine',
+    'DESCRIPTION': """
         *{command}* imports Virtual Disk images, such as VMWare VMDK files
-        and VHD files, into Google Compute Engine.
+        and VHD files, into Compute Engine.
 
-        Importing images involves 3 steps:
-        *  Upload the virtual disk file to Google Cloud Storage.
-        *  Import the image to Google Compute Engine.
+        Importing images involves three steps:
+        *  Upload the virtual disk file to Cloud Storage.
+        *  Import the image to Compute Engine.
         *  Translate the image to make a bootable image.
-        This command will perform all three of these steps as necessary,
-        depending on the input arguments specified by the user.
+        This command performs all three of these steps as required,
+        depending on the input arguments specified.
 
         This command uses the `--os` flag to choose the appropriate translation.
         You can omit the translation step using the `--data-disk` flag.
 
-        If you exported your disk from Google Compute Engine then you do not
-        need to re-import it. Instead, use the `create` command to create
-        further images from it.
+        If you exported your disk from Compute Engine then you don't
+        need to re-import it. Instead, use `{parent_command} create`
+        to create more images from the disk.
 
         Files stored on Cloud Storage and images in Compute Engine incur
         charges. See [](https://cloud.google.com/compute/docs/images/importing-virtual-disks#resource_cleanup).
         """,
+
+    'EXAMPLES': """
+        To import a centos-7 VMDK file, run:
+
+          $ {command} myimage-name --os=centos-7 --source-file=mysourcefile
+
+        To import a data disk without operating system, run:
+
+          $ {command} myimage-name --data-disk --source-file=mysourcefile
+        """,
+}
+
+ImportBeta.detailed_help = {
+    'brief': 'Import an image into Compute Engine',
+    'DESCRIPTION': """
+        *{command}* imports Virtual Disk images, such as VMWare VMDK files
+        and VHD files, into Compute Engine.
+
+        Importing images involves four steps:
+        *  Upload the virtual disk file to Cloud Storage.
+        *  Import the image to Compute Engine.
+        *  Detect the OS and bootloader contained on the disk.
+        *  Translate the image to make a bootable image.
+        This command performs all four of these steps as required,
+        depending on the input arguments specified.
+
+        To override the detected OS, specify the `--os` flag.
+        You can omit the translation step using the `--data-disk` flag.
+
+        If you exported your disk from Compute Engine then you don't
+        need to re-import it. Instead, use `{parent_command} create`
+        to create more images from the disk.
+
+        Files stored on Cloud Storage and images in Compute Engine incur
+        charges. See [](https://cloud.google.com/compute/docs/images/importing-virtual-disks#resource_cleanup).
+        """,
+
+    'EXAMPLES': """
+        To import a centos-7 VMDK file, run:
+
+          $ {command} myimage-name --os=centos-7 --source-file=mysourcefile
+
+        To import a data disk without operating system, run:
+
+          $ {command} myimage-name --data-disk --source-file=mysourcefile
+        """,
 }
