@@ -3,14 +3,14 @@
 
 # for a description of the variables, please have a look at the
 # Glossary file, as written in the Porting folder, or use the url:
-# http://perl5.git.perl.org/perl.git/blob/HEAD:/Porting/Glossary
+# https://github.com/Perl/perl5/blob/blead/Porting/Glossary
 
 package Config;
 use strict;
 use warnings;
-use vars '%Config', '$VERSION';
+our ( %Config, $VERSION );
 
-$VERSION = "5.026002";
+$VERSION = "5.032000";
 
 # Skip @Config::EXPORT because it only contains %Config, which we special
 # case below as it's not a function. @Config::EXPORT won't change in the
@@ -56,11 +56,11 @@ sub import {
     return;
 }
 
-die "$0: Perl lib version (5.26.2) doesn't match executable '$^X' version ($])"
+die "$0: Perl lib version (5.32.0) doesn't match executable '$^X' version ($])"
     unless $^V;
 
-$^V eq 5.26.2
-    or die sprintf "%s: Perl lib version (5.26.2) doesn't match executable '$^X' version (%vd)", $0, $^V;
+$^V eq 5.32.0
+    or die sprintf "%s: Perl lib version (5.32.0) doesn't match executable '$^X' version (%vd)", $0, $^V;
 
 
 sub FETCH {
@@ -84,7 +84,7 @@ sub AUTOLOAD {
 
 # tie returns the object, so the value returned to require will be true.
 tie %Config, 'Config', {
-    archlibexp => '/usr/local/lib/perl5/5.26/mach',
+    archlibexp => '/usr/local/lib/perl5/5.32/mach',
     archname => 'amd64-freebsd-thread-multi',
     cc => 'cc',
     d_readlink => 'define',
@@ -96,16 +96,16 @@ tie %Config, 'Config', {
     inc_version_list => ' ',
     intsize => '4',
     ldlibpthname => 'LD_LIBRARY_PATH',
-    libpth => '/usr/lib /usr/local/lib /usr/bin/../lib/clang/4.0.0/lib /usr/lib',
+    libpth => '/usr/lib /usr/local/lib /usr/lib/clang/8.0.1/lib /usr/lib',
     osname => 'freebsd',
-    osvers => '11.1-release-p11',
+    osvers => '12.1--hbsd',
     path_sep => ':',
-    privlibexp => '/usr/local/lib/perl5/5.26',
+    privlibexp => '/usr/local/lib/perl5/5.32',
     scriptdir => '/usr/local/bin',
-    sitearchexp => '/usr/local/lib/perl5/site_perl/mach/5.26',
+    sitearchexp => '/usr/local/lib/perl5/site_perl/mach/5.32',
     sitelibexp => '/usr/local/lib/perl5/site_perl',
     so => 'so',
     useithreads => 'define',
     usevendorprefix => undef,
-    version => '5.26.2',
+    version => '5.32.0',
 };
