@@ -58,6 +58,33 @@ class FirewallController extends ApiControllerBase
         }
     }
 
+    /**
+     * retrieve firewall log filter choices
+     * @return array
+     */
+    public function log_filtersAction()
+    {
+        $config = Config::getInstance()->object();
+        $interfaces = [];
+        if ($config->interfaces->count() > 0) {
+            foreach ($config->interfaces->children() as $key => $node) {
+                // XXX: Omit group types since they don't link to actual interfaces.
+                if (isset($node->type) && (string)$node->type == 'group') {
+                    continue;
+                } elseif ((string)$node->if == 'openvpn') {
+                    continue;
+                }
+                $interfaces[] = !empty((string)$node->descr) ? (string)$node->descr : $key;
+            }
+        }
+        sort($interfaces, SORT_NATURAL | SORT_FLAG_CASE);
+        return [
+            "interface_name" => $interfaces,
+            "dir" => ["in", "out"],
+            "action" => ["pass", "block"]
+        ];
+    }
+
     /**
      * retrieve firewall stats
      * @return array
