@@ -118,7 +118,8 @@ abstract class Base
         foreach ($configObj->system->children() as $key => $value) {
             if ($key == 'user' && !empty($value->name)) {
                 // depending on caseInSensitiveUsernames setting match exact or case-insensitive
-                if ((string)$value->name == $username ||
+                if (
+                    (string)$value->name == $username ||
                     ($this->caseInSensitiveUsernames && strtolower((string)$value->name) == strtolower($username))
                 ) {
                     // user found, stop search
@@ -129,4 +130,22 @@ abstract class Base
         }
         return $userObject;
     }
+
+    /**
+     * return actual username.
+     * This is more or less a temporary function to support case insensitive names in sessions
+     * @param string $username username
+     * @return string
+     */
+    public function getUserName($username)
+    {
+        if ($this->caseInSensitiveUsernames) {
+            $user = $this->getUser($username);
+            if ($user) {
+                return (string)$user->name;
+            }
+        } else {
+            return $username;
+        }
+    }
 }
