@@ -93,7 +93,7 @@ function saveFormToEndpoint(url, formid, callback_ok, disable_dialog, callback_f
                 if ( callback_fail !== undefined ) {
                     // execute callback function
                     callback_fail(data);
-		}
+                }
             } else if ( callback_ok !== undefined ) {
                 // execute callback function
                 callback_ok(data);
@@ -159,7 +159,7 @@ function updateServiceStatusUI(status)
         status_html += 'hidden';
     }
 
-    status_html += '"><i class="fa fa-play fa-fw"/></span>';
+    status_html += '"><i class="fa fa-play fa-fw"></i></span>';
 
     $('#service_status_container').html(status_html);
 }
@@ -169,6 +169,10 @@ function updateServiceStatusUI(status)
  */
 function updateServiceControlUI(serviceName)
 {
+    if (serviceName == '') {
+        return;
+    }
+
     ajaxCall("/api/" + serviceName + "/service/status", {}, function(data) {
         let status_html = '<span class="label label-opnsense label-opnsense-sm ';
         let status_icon = '';
@@ -187,10 +191,23 @@ function updateServiceControlUI(serviceName)
             status_html += 'hidden';
         }
 
-        status_html += '"><i class="fa fa-' + status_icon + ' fa-fw"/></span>';
+        status_html += '"><i class="fa fa-' + status_icon + ' fa-fw"></i></span>';
 
         $('#service_status_container').html(status_html + " " + buttons);
 
+        if (data['widget'] !== undefined) {
+            // tooltip service action widgets
+            ['stop', 'start', 'restart'].forEach(function(action){
+                let obj = $("#" + action + "Service");
+                if (obj.length > 0) {
+                    obj.tooltip({
+                        'placement': 'bottom',
+                        'title': data['widget']['caption_' + action]
+                    });
+                }
+            });
+        }
+
         const commands = ["start", "restart", "stop"];
         commands.forEach(function(command) {
             $("#" + command + "Service").click(function(){
@@ -261,7 +278,7 @@ function formatTokenizersUI() {
             // re-init tokenizer items
             sender.tokenize2().trigger('tokenize:clear');
             for (let i=0 ; i < items.length ; ++i) {
-              sender.tokenize2().trigger('tokenize:tokens:add', items[i]);
+                sender.tokenize2().trigger('tokenize:tokens:add', items[i]);
             }
             sender.tokenize2().trigger('tokenize:select');
             sender.tokenize2().trigger('tokenize:dropdown:hide');
@@ -367,7 +384,8 @@ function initFormAdvancedUI() {
         elements.toggleClass("text-success text-danger");
     } else {
         $('[data-advanced*="true"]').hide(function(){
-            $('[data-advanced*="true"]').after("<tr data-advanced='hidden_row'></tr>"); // the table row is added to keep correct table striping
+            // the table row is added to keep correct table striping
+            $(this).after("<tr data-advanced='hidden_row'></tr>");
         });
     }
 
@@ -458,5 +476,62 @@ function stdDialogConfirm(title, message, accept, decline, callback, type) {
  * wrapper for backwards compatibility (do not use)
  */
 function stdDialogRemoveItem(message, callback) {
-    stdDialogConfirm('Remove', message, 'Yes', 'Cancel', callback);
+    stdDialogConfirm(
+        stdDialogRemoveItem.defaults.title,  message, stdDialogRemoveItem.defaults.accept,
+        stdDialogRemoveItem.defaults.decline, callback
+    );
+}
+
+stdDialogRemoveItem.defaults = {
+    'title': 'Remove',
+    'accept': 'Yes',
+    'decline': 'Cancel'
+};
+
+
+/**
+ *  Action button, expects the following data attributes on the widget
+ *      data-endpoint='/path/to/my/endpoint'
+ *      data-label="Apply text"
+ *      data-service-widget="service" (optional service widget to signal)
+ *      data-error-title="My error message"
+ */
+$.fn.SimpleActionButton = function (params) {
+    let this_button = this;
+    this.construct = function() {
+        let label_content = '<b>' + this_button.data('label') + '</b> <i class="reload_progress">';
+        this_button.html(label_content);
+        this_button.on('click', function(){
+            this_button.find('.reload_progress').addClass("fa fa-spinner fa-pulse");
+            let pre_action = function() {
+                return (new $.Deferred()).resolve();
+            }
+            if (params && params.onPreAction) {
+                pre_action = params.onPreAction;
+            }
+            pre_action().done(function() {
+                ajaxCall(this_button.data('endpoint'), {}, function(data,status) {
+                    if (params && params.onAction) {
+                        params.onAction(data, status);
+                    }
+                    if ((status != "success" || data['status'].toLowerCase().trim() != 'ok') && data['status']) {
+                          BootstrapDialog.show({
+                              type: BootstrapDialog.TYPE_WARNING,
+                              title: this_button.data('error-title'),
+                              message: data['status'],
+                              draggable: true
+                          });
+                    }
+                    this_button.find('.reload_progress').removeClass("fa fa-spinner fa-pulse");
+                    if (this_button.data('service-widget')) {
+                        updateServiceControlUI(this_button.data('service-widget'));
+                    }
+                });
+            });
+        });
+    }
+    return this.each(function(){
+        const button = this_button.construct();
+        return button;
+    });
 }
