@@ -1,14 +1,12 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 
 """
-Copyright (c) 2014-2019 Maltrail developers (https://github.com/stamparm/maltrail/)
+Copyright (c) 2014-2021 Maltrail developers (https://github.com/stamparm/maltrail/)
 See the file 'LICENSE' for copying permission
 """
+from __future__ import print_function
 
-import BaseHTTPServer
-import cStringIO
 import datetime
-import httplib
 import glob
 import gzip
 import hashlib
@@ -18,13 +16,10 @@ import mimetypes
 import os
 import re
 import socket
-import SocketServer
 import subprocess
 import threading
 import time
 import traceback
-import urllib
-import urlparse
 
 from core.addr import addr_to_int
 from core.addr import int_to_addr
@@ -33,6 +28,7 @@ from core.attribdict import AttribDict
 from core.common import get_regex
 from core.common import ipcat_lookup
 from core.common import worst_asns
+from core.compat import xrange
 from core.enums import HTTP_HEADER
 from core.settings import config
 from core.settings import CONTENT_EXTENSIONS_EXCLUSIONS
@@ -41,17 +37,23 @@ from core.settings import DISABLED_CONTENT_EXTENSIONS
 from core.settings import DISPOSED_NONCES
 from core.settings import HTML_DIR
 from core.settings import HTTP_TIME_FORMAT
+from core.settings import IS_WIN
 from core.settings import MAX_NOFILE
 from core.settings import NAME
 from core.settings import PING_RESPONSE
-from core.settings import SERVER_HEADER
 from core.settings import SESSION_COOKIE_NAME
 from core.settings import SESSION_COOKIE_FLAG_SAMESITE
 from core.settings import SESSION_EXPIRATION_HOURS
 from core.settings import SESSION_ID_LENGTH
 from core.settings import SESSIONS
 from core.settings import UNAUTHORIZED_SLEEP_TIME
+from core.settings import UNICODE_ENCODING
 from core.settings import VERSION
+from thirdparty import six
+from thirdparty.six.moves import BaseHTTPServer as _BaseHTTPServer
+from thirdparty.six.moves import http_client as _http_client
+from thirdparty.six.moves import socketserver as _socketserver
+from thirdparty.six.moves import urllib as _urllib
 
 try:
     # Reference: https://bugs.python.org/issue7980
@@ -66,19 +68,22 @@ try:
 except:
     pass
 
+_fail2ban_cache = None
+_fail2ban_key = None
+
 def start_httpd(address=None, port=None, join=False, pem=None):
     """
     Starts HTTP server
     """
 
-    class ThreadingServer(SocketServer.ThreadingMixIn, BaseHTTPServer.HTTPServer):
+    class ThreadingServer(_socketserver.ThreadingMixIn, _BaseHTTPServer.HTTPServer):
         def server_bind(self):
             self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-            BaseHTTPServer.HTTPServer.server_bind(self)
+            _BaseHTTPServer.HTTPServer.server_bind(self)
 
         def finish_request(self, *args, **kwargs):
             try:
-                BaseHTTPServer.HTTPServer.finish_request(self, *args, **kwargs)
+                _BaseHTTPServer.HTTPServer.finish_request(self, *args, **kwargs)
             except:
                 if config.SHOW_DEBUG:
                     traceback.print_exc()
@@ -88,7 +93,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
             import OpenSSL  # python-openssl
 
             ThreadingServer.__init__(self, server_address, HandlerClass)
-            ctx = OpenSSL.SSL.Context(OpenSSL.SSL.TLSv1_METHOD)
+            ctx = OpenSSL.SSL.Context(OpenSSL.SSL.TLSv1_2_METHOD)
             ctx.use_privatekey_file(pem)
             ctx.use_certificate_file(pem)
             self.socket = OpenSSL.SSL.Connection(ctx, socket.socket(self.address_family, self.socket_type))
@@ -99,10 +104,9 @@ def start_httpd(address=None, port=None, join=False, pem=None):
             try:
                 request.shutdown()
             except:
-                if config.SHOW_DEBUG:
-                    traceback.print_exc()
+                pass
 
-    class ReqHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+    class ReqHandler(_BaseHTTPServer.BaseHTTPRequestHandler):
         def do_GET(self):
             path, query = self.path.split('?', 1) if '?' in self.path else (self.path, "")
             params = {}
@@ -110,10 +114,10 @@ def start_httpd(address=None, port=None, join=False, pem=None):
             skip = False
 
             if hasattr(self, "data"):
-                params.update(urlparse.parse_qs(self.data))
+                params.update(_urllib.parse.parse_qs(self.data))
 
             if query:
-                params.update(urlparse.parse_qs(query))
+                params.update(_urllib.parse.parse_qs(query))
 
             for key in params:
                 if params[key]:
@@ -138,28 +142,29 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                 if any((config.IP_ALIASES,)) and self.path.split('?')[0] == "/js/main.js":
                     content = open(path, "rb").read()
                     content = re.sub(r"\bvar IP_ALIASES =.+", "var IP_ALIASES = {%s};" % ", ".join('"%s": "%s"' % (_.split(':', 1)[0].strip(), _.split(':', 1)[-1].strip()) for _ in config.IP_ALIASES), content)
-                    self.send_response(httplib.OK)
-                elif ".." not in os.path.relpath(path, HTML_DIR) and os.path.isfile(path) and (extension not in DISABLED_CONTENT_EXTENSIONS or os.path.split(path)[-1] in CONTENT_EXTENSIONS_EXCLUSIONS):
+
+                if ".." not in os.path.relpath(path, HTML_DIR) and os.path.isfile(path) and (extension not in DISABLED_CONTENT_EXTENSIONS or os.path.split(path)[-1] in CONTENT_EXTENSIONS_EXCLUSIONS):
                     mtime = time.gmtime(os.path.getmtime(path))
                     if_modified_since = self.headers.get(HTTP_HEADER.IF_MODIFIED_SINCE)
 
                     if if_modified_since and extension not in (".htm", ".html"):
                         if_modified_since = [_ for _ in if_modified_since.split(';') if _.upper().endswith("GMT")][0]
                         if time.mktime(mtime) <= time.mktime(time.strptime(if_modified_since, HTTP_TIME_FORMAT)):
-                            self.send_response(httplib.NOT_MODIFIED)
+                            self.send_response(_http_client.NOT_MODIFIED)
                             self.send_header(HTTP_HEADER.CONNECTION, "close")
                             skip = True
 
                     if not skip:
-                        content = open(path, "rb").read()
+                        content = content or open(path, "rb").read()
                         last_modified = time.strftime(HTTP_TIME_FORMAT, mtime)
-                        self.send_response(httplib.OK)
+                        self.send_response(_http_client.OK)
                         self.send_header(HTTP_HEADER.CONNECTION, "close")
                         self.send_header(HTTP_HEADER.CONTENT_TYPE, mimetypes.guess_type(path)[0] or "application/octet-stream")
                         self.send_header(HTTP_HEADER.LAST_MODIFIED, last_modified)
-                        self.send_header(HTTP_HEADER.CONTENT_SECURITY_POLICY, "default-src 'none'; style-src 'self' 'unsafe-inline'; img-src 'self'; " +
-                                                                              "script-src 'self' 'unsafe-eval' https://stat.ripe.net; connect-src 'self'; form-action 'self'; " +
-                                                                              "font-src 'self'; frame-src *; worker-src 'self'; block-all-mixed-content;")
+
+                        # For CSP policy directives see: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy/
+                        self.send_header(HTTP_HEADER.CONTENT_SECURITY_POLICY, "default-src 'self'; style-src 'self' 'unsafe-inline'; img-src * blob:; script-src 'self' 'unsafe-eval' https://stat.ripe.net; frame-src *; object-src 'none'; block-all-mixed-content;")
+
                         if extension not in (".htm", ".html"):
                             self.send_header(HTTP_HEADER.EXPIRES, "Sun, 17-Jan-2038 19:14:07 GMT")        # Reference: http://blog.httpwatch.com/2007/12/10/two-simple-rules-for-http-caching/
                             self.send_header(HTTP_HEADER.CACHE_CONTROL, "max-age=3600, must-revalidate")  # Reference: http://stackoverflow.com/a/5084555
@@ -167,20 +172,23 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                             self.send_header(HTTP_HEADER.CACHE_CONTROL, "no-cache")
 
                 else:
-                    self.send_response(httplib.NOT_FOUND)
+                    self.send_response(_http_client.NOT_FOUND)
                     self.send_header(HTTP_HEADER.CONNECTION, "close")
-                    content = '<!DOCTYPE html><html lang="en"><head><title>404 Not Found</title></head><body><h1>Not Found</h1><p>The requested URL %s was not found on this server.</p></body></html>' % self.path.split('?')[0]
+                    content = b'<!DOCTYPE html><html lang="en"><head><title>404 Not Found</title></head><body><h1>Not Found</h1><p>The requested URL %s was not found on this server.</p></body></html>' % self.path.split('?')[0]
 
             if content is not None:
-                for match in re.finditer(r"<\!(\w+)\!>", content):
-                    name = match.group(1)
+                if isinstance(content, six.text_type):
+                    content = content.encode(UNICODE_ENCODING)
+
+                for match in re.finditer(b"<\\!(\\w+)\\!>", content):
+                    name = match.group(1).decode(UNICODE_ENCODING)
                     _ = getattr(self, "_%s" % name.lower(), None)
                     if _:
-                        content = self._format(content, **{ name: _() })
+                        content = self._format(content, **{name: _()})
 
-                if "gzip" in self.headers.getheader(HTTP_HEADER.ACCEPT_ENCODING, ""):
+                if "gzip" in self.headers.get(HTTP_HEADER.ACCEPT_ENCODING, ""):
                     self.send_header(HTTP_HEADER.CONTENT_ENCODING, "gzip")
-                    _ = cStringIO.StringIO()
+                    _ = six.BytesIO()
                     compress = gzip.GzipFile("", "w+b", 9, _)
                     compress._stream = _
                     compress.write(content)
@@ -192,16 +200,18 @@ def start_httpd(address=None, port=None, join=False, pem=None):
 
             self.end_headers()
 
-            if content:
-                self.wfile.write(content)
+            try:
+                if content:
+                    self.wfile.write(content)
 
-            self.wfile.flush()
-            self.wfile.close()
+                self.wfile.flush()
+            except:
+                pass
 
         def do_POST(self):
-            length = self.headers.getheader(HTTP_HEADER.CONTENT_LENGTH)
-            data = self.rfile.read(int(length))
-            data = urllib.unquote_plus(data)
+            length = self.headers.get(HTTP_HEADER.CONTENT_LENGTH)
+            data = self.rfile.read(int(length)).decode(UNICODE_ENCODING)
+            data = _urllib.parse.unquote_plus(data)
             self.data = data
             self.do_GET()
 
@@ -237,11 +247,11 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                         del SESSIONS[session]
 
         def version_string(self):
-            return SERVER_HEADER
+            return "%s/%s" % (NAME, self._version())
 
         def end_headers(self):
             if not hasattr(self, "_headers_ended"):
-                BaseHTTPServer.BaseHTTPRequestHandler.end_headers(self)
+                _BaseHTTPServer.BaseHTTPRequestHandler.end_headers(self)
                 self._headers_ended = True
 
         def log_message(self, format, *args):
@@ -249,18 +259,41 @@ def start_httpd(address=None, port=None, join=False, pem=None):
 
         def finish(self):
             try:
-                BaseHTTPServer.BaseHTTPRequestHandler.finish(self)
+                _BaseHTTPServer.BaseHTTPRequestHandler.finish(self)
             except:
                 if config.SHOW_DEBUG:
                     traceback.print_exc()
 
         def _version(self):
-            return VERSION
+            version = VERSION
+
+            try:
+                for line in open(os.path.join(os.path.dirname(__file__), "settings.py"), 'r'):
+                    match = re.search(r'VERSION = "([^"]*)', line)
+                    if match:
+                        version = match.group(1)
+                        break
+            except:
+                pass
+
+            return version
+
+        def _statics(self):
+            latest = max(glob.glob(os.path.join(os.path.dirname(__file__), "..", "trails", "static", "malware", "*.txt")), key=os.path.getmtime)
+            return "/%s" % datetime.datetime.fromtimestamp(os.path.getmtime(latest)).strftime(DATE_FORMAT)
+
+        def _logo(self):
+            if config.HEADER_LOGO:
+                retval = config.HEADER_LOGO
+            else:
+                retval = '<img src="images/mlogo.png" style="width: 25px">altrail'
+
+            return retval
 
         def _format(self, content, **params):
             if content:
                 for key, value in params.items():
-                    content = content.replace("<!%s!>" % key, value)
+                    content = content.replace(b"<!%s!>" % key.encode(UNICODE_ENCODING), value.encode(UNICODE_ENCODING))
 
             return content
 
@@ -273,9 +306,15 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                     for entry in (config.USERS or []):
                         entry = re.sub(r"\s", "", entry)
                         username, stored_hash, uid, netfilter = entry.split(':')
+
+                        try:
+                            uid = int(uid)
+                        except ValueError:
+                            uid = None
+
                         if username == params.get("username"):
                             try:
-                                if params.get("hash") == hashlib.sha256(stored_hash.strip() + params.get("nonce")).hexdigest():
+                                if params.get("hash") == hashlib.sha256((stored_hash.strip() + params.get("nonce")).encode(UNICODE_ENCODING)).hexdigest():
                                     valid = True
                                     break
                             except:
@@ -283,10 +322,11 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                                     traceback.print_exc()
 
             if valid:
-                session_id = os.urandom(SESSION_ID_LENGTH).encode("hex")
+                _ = os.urandom(SESSION_ID_LENGTH)
+                session_id = _.hex() if hasattr(_, "hex") else _.encode("hex")
                 expiration = time.time() + 3600 * SESSION_EXPIRATION_HOURS
 
-                self.send_response(httplib.OK)
+                self.send_response(_http_client.OK)
                 self.send_header(HTTP_HEADER.CONNECTION, "close")
 
                 cookie = "%s=%s; expires=%s; path=/; HttpOnly" % (SESSION_COOKIE_NAME, session_id, time.strftime(HTTP_TIME_FORMAT, time.gmtime(expiration)))
@@ -328,16 +368,16 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                     if addresses:
                         netfilters.add(get_regex(addresses))
 
-                SESSIONS[session_id] = AttribDict({"username": username, "uid": uid, "netfilters": netfilters, "expiration": expiration, "client_ip": self.client_address[0]})
+                SESSIONS[session_id] = AttribDict({"username": username, "uid": uid, "netfilters": netfilters, "mask_custom": config.ENABLE_MASK_CUSTOM and uid >= 1000, "expiration": expiration, "client_ip": self.client_address[0]})
             else:
                 time.sleep(UNAUTHORIZED_SLEEP_TIME)
-                self.send_response(httplib.UNAUTHORIZED)
+                self.send_response(_http_client.UNAUTHORIZED)
                 self.send_header(HTTP_HEADER.CONNECTION, "close")
 
             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
             content = "Login %s" % ("success" if valid else "failed")
 
-            if not subprocess.mswindows:
+            if not IS_WIN:
                 try:
                     subprocess.check_output("logger -p auth.info -t \"%s[%d]\" \"%s password for %s from %s port %s\"" % (NAME.lower(), os.getpid(), "Accepted" if valid else "Failed", params.get("username"), self.client_address[0], self.client_address[1]), stderr=subprocess.STDOUT, shell=True)
                 except Exception:
@@ -348,7 +388,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
 
         def _logout(self, params):
             self.delete_session()
-            self.send_response(httplib.FOUND)
+            self.send_response(_http_client.FOUND)
             self.send_header(HTTP_HEADER.CONNECTION, "close")
             self.send_header(HTTP_HEADER.LOCATION, "/")
 
@@ -356,7 +396,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
             session = self.get_session()
             username = session.username if session else ""
 
-            self.send_response(httplib.OK)
+            self.send_response(_http_client.OK)
             self.send_header(HTTP_HEADER.CONNECTION, "close")
             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
 
@@ -366,11 +406,11 @@ def start_httpd(address=None, port=None, join=False, pem=None):
             session = self.get_session()
 
             if session is None:
-                self.send_response(httplib.UNAUTHORIZED)
+                self.send_response(_http_client.UNAUTHORIZED)
                 self.send_header(HTTP_HEADER.CONNECTION, "close")
                 return None
 
-            self.send_response(httplib.OK)
+            self.send_response(_http_client.OK)
             self.send_header(HTTP_HEADER.CONNECTION, "close")
             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
 
@@ -387,24 +427,60 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                     traceback.print_exc()
 
         def _trails(self, params):
-            self.send_response(httplib.OK)
+            self.send_response(_http_client.OK)
             self.send_header(HTTP_HEADER.CONNECTION, "close")
             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
 
             return open(config.TRAILS_FILE, "rb").read()
 
         def _ping(self, params):
-            self.send_response(httplib.OK)
+            self.send_response(_http_client.OK)
             self.send_header(HTTP_HEADER.CONNECTION, "close")
             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
 
             return PING_RESPONSE
 
+        def _fail2ban(self, params):
+            global _fail2ban_cache
+            global _fail2ban_key
+
+            self.send_response(_http_client.OK)
+            self.send_header(HTTP_HEADER.CONNECTION, "close")
+            self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
+
+            content = ""
+            key = int(time.time()) >> 3
+
+            if config.FAIL2BAN_REGEX:
+                try:
+                    re.compile(config.FAIL2BAN_REGEX)
+                except re.error:
+                    content = "invalid regular expression used in option FAIL2BAN_REGEX"
+                else:
+                    if key == _fail2ban_key:
+                        content = _fail2ban_cache
+                    else:
+                        result = set()
+                        _ = os.path.join(config.LOG_DIR, "%s.log" % datetime.datetime.now().strftime("%Y-%m-%d"))
+                        if os.path.isfile(_):
+                            for line in open(_, "r"):
+                                if re.search(config.FAIL2BAN_REGEX, line, re.I):
+                                    result.add(line.split()[3])
+
+                        content = "\n".join(result)
+
+                        _fail2ban_cache = content
+                        _fail2ban_key = key
+            else:
+                content = "configuration option FAIL2BAN_REGEX not set"
+
+            return content
+
         def _events(self, params):
             session = self.get_session()
 
             if session is None:
-                self.send_response(httplib.UNAUTHORIZED)
+                self.send_response(_http_client.UNAUTHORIZED)
                 self.send_header(HTTP_HEADER.CONNECTION, "close")
                 return None
 
@@ -423,7 +499,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                         range_handle = open(event_log_path, "rb")
                         log_exists = True
                 except ValueError:
-                    print "[!] invalid date format in request"
+                    print("[!] invalid date format in request")
                     log_exists = False
             else:
                 logs_data = ""
@@ -442,7 +518,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                     range_handle = io.BytesIO(logs_data)
                     log_exists = True
                 except ValueError:
-                    print "[!] invalid date format in request"
+                    print("[!] invalid date format in request")
                     log_exists = False
 
             if log_exists:
@@ -461,35 +537,36 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                         if start == 0 or not session.range_handle:
                             session.range_handle = range_handle
 
-                        if session.netfilters is None:
+                        if session.netfilters is None and not session.mask_custom:
                             session.range_handle.seek(start)
-                            self.send_response(httplib.PARTIAL_CONTENT)
+                            self.send_response(_http_client.PARTIAL_CONTENT)
                             self.send_header(HTTP_HEADER.CONNECTION, "close")
                             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
                             self.send_header(HTTP_HEADER.CONTENT_RANGE, "bytes %d-%d/%d" % (start, end, total))
                             content = session.range_handle.read(size)
                         else:
-                            self.send_response(httplib.OK)
+                            self.send_response(_http_client.OK)
                             self.send_header(HTTP_HEADER.CONNECTION, "close")
                             self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
 
-                            buffer, addresses, netmasks, regex = cStringIO.StringIO(), set(), [], ""
-                            for netfilter in session.netfilters:
+                            buffer, addresses, netmasks, regex = io.StringIO(), set(), [], ""
+                            for netfilter in session.netfilters or []:
                                 if not netfilter:
                                     continue
                                 if '/' in netfilter:
                                     netmasks.append(netfilter)
                                 elif re.search(r"\A[\d.]+\Z", netfilter):
                                     addresses.add(netfilter)
-                                elif '\.' in netfilter:
+                                elif "\\." in netfilter:
                                     regex = r"\b(%s)\b" % netfilter
                                 else:
-                                    print "[!] invalid network filter '%s'" % netfilter
+                                    print("[!] invalid network filter '%s'" % netfilter)
                                     return
 
                             for line in session.range_handle:
-                                display = False
+                                display = session.netfilters is None
                                 ip = None
+                                line = line.decode(UNICODE_ENCODING, "ignore")
 
                                 if regex:
                                     match = re.search(regex, line)
@@ -515,6 +592,9 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                                                     display = True
                                                     break
 
+                                if session.mask_custom and "(custom)" in line:
+                                    line = re.sub(r'("[^"]+"|[^ ]+) \(custom\)', "- (custom)", line)
+
                                 if display:
                                     if ",%s" % ip in line or "%s," % ip in line:
                                         line = re.sub(r" ([\d.,]+,)?%s(,[\d.,]+)? " % re.escape(ip), " %s " % ip, line)
@@ -531,7 +611,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                             session.range_handle = None
 
                 if size == -1:
-                    self.send_response(httplib.OK)
+                    self.send_response(_http_client.OK)
                     self.send_header(HTTP_HEADER.CONNECTION, "close")
                     self.send_header(HTTP_HEADER.CONTENT_TYPE, "text/plain")
                     self.end_headers()
@@ -545,7 +625,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                                 self.wfile.write(data)
 
             else:
-                self.send_response(httplib.OK)  # instead of httplib.NO_CONTENT (compatibility reasons)
+                self.send_response(_http_client.OK)  # instead of _http_client.NO_CONTENT (compatibility reasons)
                 self.send_header(HTTP_HEADER.CONNECTION, "close")
                 if self.headers.get(HTTP_HEADER.RANGE):
                     self.send_header(HTTP_HEADER.CONTENT_RANGE, "bytes 0-0/0")
@@ -558,11 +638,11 @@ def start_httpd(address=None, port=None, join=False, pem=None):
             session = self.get_session()
 
             if session is None:
-                self.send_response(httplib.UNAUTHORIZED)
+                self.send_response(_http_client.UNAUTHORIZED)
                 self.send_header(HTTP_HEADER.CONNECTION, "close")
                 return None
 
-            self.send_response(httplib.OK)
+            self.send_response(_http_client.OK)
             self.send_header(HTTP_HEADER.CONNECTION, "close")
             self.send_header(HTTP_HEADER.CONTENT_TYPE, "application/json")
 
@@ -597,10 +677,10 @@ def start_httpd(address=None, port=None, join=False, pem=None):
                         with open(filepath, "rb") as f:
                             content = f.read(io.DEFAULT_BUFFER_SIZE)
                             if size >= io.DEFAULT_BUFFER_SIZE:
-                                total = 1.0 * content.count('\n') * size / io.DEFAULT_BUFFER_SIZE
-                                counts[timestamp] = int(round(total / 100) * 100)
+                                total = 1.0 * (1 + content.count(b'\n')) * size / io.DEFAULT_BUFFER_SIZE
+                                counts[timestamp] = int(round(total / 100.0) * 100)
                             else:
-                                counts[timestamp] = content.count('\n')
+                                counts[timestamp] = content.count(b'\n')
 
             return json.dumps(counts)
 
@@ -614,7 +694,7 @@ def start_httpd(address=None, port=None, join=False, pem=None):
     if ':' in (address or ""):
         address = address.strip("[]")
 
-        BaseHTTPServer.HTTPServer.address_family = socket.AF_INET6
+        _BaseHTTPServer.HTTPServer.address_family = socket.AF_INET6
 
         # Reference: https://github.com/squeaky-pl/zenchmarks/blob/master/vendor/twisted/internet/tcp.py
         _AI_NUMERICSERV = getattr(socket, "AI_NUMERICSERV", 0)
@@ -639,9 +719,9 @@ def start_httpd(address=None, port=None, join=False, pem=None):
         else:
             raise
 
-    print "[i] starting HTTP%s server at 'http%s://%s:%d/'" % ('S' if pem else "", 's' if pem else "", server.server_address[0], server.server_address[1])
+    print("[i] starting HTTP%s server at http%s://%s:%d/" % ('S' if pem else "", 's' if pem else "", server.server_address[0], server.server_address[1]))
 
-    print "[o] running..."
+    print("[o] running...")
 
     if join:
         server.serve_forever()
