@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2014-2020 Maltrail developers (https://github.com/stamparm/maltrail/)
+Copyright (c) 2014-2021 Maltrail developers (https://github.com/stamparm/maltrail/)
 See the file 'LICENSE' for copying permission
 """
 
@@ -25,6 +25,7 @@ import struct
 import threading
 import time
 import traceback
+import warnings
 
 from core.addr import inet_ntoa6
 from core.addr import addr_port
@@ -91,7 +92,6 @@ from core.settings import SUSPICIOUS_UA_REGEX
 from core.settings import VALID_DNS_NAME_REGEX
 from core.settings import trails
 from core.settings import VERSION
-from core.settings import WEB_SHELLS
 from core.settings import WHITELIST
 from core.settings import WHITELIST_DIRECT_DOWNLOAD_KEYWORDS
 from core.settings import WHITELIST_LONG_DOMAIN_NAME_KEYWORDS
@@ -102,6 +102,8 @@ from core.update import update_trails
 from thirdparty import six
 from thirdparty.six.moves import urllib as _urllib
 
+warnings.filterwarnings(action="ignore", category=DeprecationWarning)       # NOTE: https://github.com/helpsystems/pcapy/pull/67/files
+
 _buffer = None
 _caps = []
 _connect_sec = 0
@@ -117,13 +119,15 @@ _last_syn = None
 _last_logged_syn = None
 _last_udp = None
 _last_logged_udp = None
-_last_dns_exhaustion = None
 _done_count = 0
 _done_lock = threading.Lock()
 _subdomains = {}
 _subdomains_sec = None
 _dns_exhausted_domains = set()
 
+class _set(set):
+    pass
+
 try:
     import pcapy
 except ImportError:
@@ -162,41 +166,14 @@ def _check_domain(query, sec, usec, src_ip, src_port, dst_ip, dst_port, proto, p
     if query.replace('.', "").isdigit():  # IP address
         return
 
-    if _result_cache.get((CACHE_TYPE.DOMAIN, query)) == False:
+    if _result_cache.get((CACHE_TYPE.DOMAIN, query)) is False:
         return
 
     result = False
     if re.search(VALID_DNS_NAME_REGEX, query) is not None and not _check_domain_whitelisted(query):
         parts = query.split('.')
 
-        if trails._regex:
-            match = re.search(trails._regex, query)
-            if match:
-                group, trail = [_ for _ in match.groupdict().items() if _[1] is not None][0]
-                candidate = trails._regex.split("(?P<")[int(group[1:]) + 1]
-                candidate = candidate.split('>', 1)[-1].rstrip('|')[:-1]
-                if candidate in trails:
-                    result = True
-                    trail = match.group(0)
-
-                    prefix, suffix = query[:match.start()], query[match.end():]
-                    if prefix:
-                        trail = "(%s)%s" % (prefix, trail)
-                    if suffix:
-                        trail = "%s(%s)" % (trail, suffix)
-
-                    trail = trail.replace(".)", ").")
-
-                    log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, proto, TRAIL.DNS, trail, trails[candidate][0], trails[candidate][1]), packet)
-
-        if ".onion." in query:
-            trail = re.sub(r"(\.onion)(\..*)", r"\1(\2)", query)
-            _ = trail.split('(')[0]
-            if _ in trails:
-                result = True
-                log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, proto, TRAIL.DNS, trail, trails[_][0], trails[_][1]), packet)
-
-        elif query.endswith(".ip-adress.com"):  # Reference: https://www.virustotal.com/gui/domain/ip-adress.com/relations
+        if query.endswith(".ip-adress.com"):  # Reference: https://www.virustotal.com/gui/domain/ip-adress.com/relations
             _ = '.'.join(parts[:-2])
             trail = "%s(.ip-adress.com)" % _
             if _ in trails:
@@ -234,7 +211,34 @@ def _check_domain(query, sec, usec, src_ip, src_port, dst_ip, dst_port, proto, p
                     result = True
                     log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, proto, TRAIL.DNS, trail, "long domain (suspicious)", "(heuristic)"), packet)
 
-    if result == False:
+        if not result and trails._regex:
+            match = re.search(trails._regex, query)
+            if match:
+                group, trail = [_ for _ in match.groupdict().items() if _[1] is not None][0]
+                candidate = trails._regex.split("(?P<")[int(group[1:]) + 1]
+                candidate = candidate.split('>', 1)[-1].rstrip('|')[:-1]
+                if candidate in trails:
+                    result = True
+                    trail = match.group(0)
+
+                    prefix, suffix = query[:match.start()], query[match.end():]
+                    if prefix:
+                        trail = "(%s)%s" % (prefix, trail)
+                    if suffix:
+                        trail = "%s(%s)" % (trail, suffix)
+
+                    trail = trail.replace(".)", ").")
+
+                    log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, proto, TRAIL.DNS, trail, trails[candidate][0], trails[candidate][1]), packet)
+
+        if not result and ".onion." in query:
+            trail = re.sub(r"(\.onion)(\..*)", r"\1(\2)", query)
+            _ = trail.split('(')[0]
+            if _ in trails:
+                result = True
+                log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, proto, TRAIL.DNS, trail, trails[_][0], trails[_][1]), packet)
+
+    if result is False:
         _result_cache[(CACHE_TYPE.DOMAIN, query)] = False
 
 def _get_local_prefix():
@@ -260,7 +264,6 @@ def _process_packet(packet, sec, usec, ip_offset):
     global _last_logged_syn
     global _last_udp
     global _last_logged_udp
-    global _last_dns_exhaustion
     global _subdomains_sec
 
     try:
@@ -318,7 +321,7 @@ def _process_packet(packet, sec, usec, ip_offset):
             return
 
         if protocol == socket.IPPROTO_TCP:  # TCP
-            src_port, dst_port, _, _, doff_reserved, flags = struct.unpack("!HHLLBB", ip_data[iph_length:iph_length+14])
+            src_port, dst_port, _, _, doff_reserved, flags = struct.unpack("!HHLLBB", ip_data[iph_length:iph_length + 14])
 
             if flags != 2 and config.plugin_functions:
                 if dst_ip in trails:
@@ -382,7 +385,7 @@ def _process_packet(packet, sec, usec, ip_offset):
                         index = tcp_data.find("<title>")
                         if index >= 0:
                             title = tcp_data[index + len("<title>"):tcp_data.find("</title>", index)]
-                            if all(_ in title.lower() for _ in ("this domain", "has been seized")):
+                            if re.search(r"domain name has been seized by|Domain Seized|Domain Seizure", title):
                                 log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.TCP, TRAIL.HTTP, title, "seized domain (suspicious)", "(heuristic)"), packet)
 
                     content_type = None
@@ -491,7 +494,7 @@ def _process_packet(packet, sec, usec, ip_offset):
                                     match = re.search(SUSPICIOUS_UA_REGEX, user_agent)
                                     if match:
                                         def _(value):
-                                            return value.replace('(', "\\(").replace(')', "\\)")
+                                            return value.rstrip('\\').replace('(', "\\(").replace(')', "\\)")
 
                                         parts = user_agent.split(match.group(0), 1)
 
@@ -506,22 +509,27 @@ def _process_packet(packet, sec, usec, ip_offset):
                                 log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.TCP, TRAIL.UA, result, "user agent (suspicious)", "(heuristic)"), packet)
 
                     if not _check_domain_whitelisted(host):
+                        path = path.replace("//", '/')
+
                         unquoted_path = _urllib.parse.unquote(path)
                         unquoted_post_data = _urllib.parse.unquote(post_data or "")
 
                         checks = [path.rstrip('/')]
+
                         if '?' in path:
                             checks.append(path.split('?')[0].rstrip('/'))
 
                             if '=' in path:
                                 checks.append(path[:path.index('=') + 1])
+
+                            _ = re.sub(r"(\w+=)[^&=]+", r"\g<1>", path)
+                            if _ not in checks:
+                                checks.append(_)
+                                if _.count('/') > 1:
+                                    checks.append("/%s" % _.split('/')[-1])
                         elif post_data:
                             checks.append("%s?%s" % (path, unquoted_post_data.lower()))
 
-                        #_ = os.path.splitext(checks[-1])       # causing FPs in cases like elf_mirai - /juno if legit /juno.php is accessed
-                        #if _[1]:
-                            #checks.append(_[0])
-
                         if checks[-1].count('/') > 1:
                             checks.append(checks[-1][:checks[-1].rfind('/')])
                             checks.append(checks[0][checks[0].rfind('/'):].split('?')[0])
@@ -566,7 +574,7 @@ def _process_packet(packet, sec, usec, ip_offset):
                                                 found = desc
                                                 break
                                         _result_cache[(CACHE_TYPE.PATH, unquoted_path)] = found or ""
-                                    if found:
+                                    if found and not ("data leakage" in found and is_local(dst_ip)):
                                         trail = "%s(%s)" % (host, path)
                                         log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.TCP, TRAIL.URL, trail, "%s (suspicious)" % found, "(heuristic)"), packet)
                                         return
@@ -590,10 +598,8 @@ def _process_packet(packet, sec, usec, ip_offset):
                                 filename = _.path.split('/')[-1]
                                 name, extension = os.path.splitext(filename)
                                 trail = "%s(%s)" % (host, path)
-                                if extension and extension in SUSPICIOUS_DIRECT_DOWNLOAD_EXTENSIONS and not any(_ in path for _ in WHITELIST_DIRECT_DOWNLOAD_KEYWORDS) and '=' not in _.query and len(name) < 10:
+                                if extension in SUSPICIOUS_DIRECT_DOWNLOAD_EXTENSIONS and not is_local(dst_ip) and not any(_ in path for _ in WHITELIST_DIRECT_DOWNLOAD_KEYWORDS) and '=' not in _.query and len(name) < 10:
                                     log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.TCP, TRAIL.URL, trail, "direct %s download (suspicious)" % extension, "(heuristic)"), packet)
-                                elif filename in WEB_SHELLS:
-                                    log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.TCP, TRAIL.URL, trail, "potential web shell (suspicious)", "(heuristic)"), packet)
                                 else:
                                     for desc, regex in SUSPICIOUS_HTTP_PATH_REGEXES:
                                         if re.search(regex, filename, re.I):
@@ -670,19 +676,21 @@ def _process_packet(packet, sec, usec, ip_offset):
                                     subdomains = _subdomains.get(domain)
 
                                     if not subdomains:
-                                        subdomains = _subdomains[domain] = set()
+                                        subdomains = _subdomains[domain] = _set()
+                                        subdomains._start = sec
 
                                     if not re.search(r"\A\d+\-\d+\-\d+\-\d+\Z", parts[0]):
-                                        if len(subdomains) < DNS_EXHAUSTION_THRESHOLD:
+                                        if sec - subdomains._start > 60:
+                                            subdomains._start = sec
+                                            subdomains.clear()
+                                        elif len(subdomains) < DNS_EXHAUSTION_THRESHOLD:
                                             subdomains.add('.'.join(parts[:-2]))
                                         else:
-                                            if (sec - (_last_dns_exhaustion or 0)) > 60:
-                                                trail = "(%s).%s" % ('.'.join(parts[:-2]), '.'.join(parts[-2:]))
-                                                if re.search(r"bl\b", trail) is None:                                               # generic check for DNSBLs
-                                                    if not any(_ in subdomains for _ in LOCAL_SUBDOMAIN_LOOKUPS):                   # generic check for local DNS resolutions
-                                                        log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.UDP, TRAIL.DNS, trail, "potential dns exhaustion (suspicious)", "(heuristic)"), packet)
-                                                        _dns_exhausted_domains.add(domain)
-                                                        _last_dns_exhaustion = sec
+                                            trail = "(%s).%s" % ('.'.join(parts[:-2]), '.'.join(parts[-2:]))
+                                            if re.search(r"bl\b", trail) is None:                                               # generic check for DNSBLs
+                                                if not any(_ in subdomains for _ in LOCAL_SUBDOMAIN_LOOKUPS):                   # generic check for local DNS resolutions
+                                                    log_event((sec, usec, src_ip, src_port, dst_ip, dst_port, PROTO.UDP, TRAIL.DNS, trail, "potential dns exhaustion (suspicious)", "(heuristic)"), packet)
+                                                    _dns_exhausted_domains.add(domain)
 
                                             return
 
@@ -847,7 +855,7 @@ def init():
                 if re.search(r"[\].][*+]|\[[a-z0-9_.\-]+\]", trail, re.I):
                     try:
                         re.compile(trail)
-                    except:
+                    except re.error:
                         pass
                     else:
                         if re.escape(trail) != trail:
@@ -954,6 +962,15 @@ def init():
     if config.SYSLOG_SERVER and not len(config.SYSLOG_SERVER.split(':')) == 2:
         exit("[!] invalid configuration value for 'SYSLOG_SERVER' ('%s')" % config.SYSLOG_SERVER)
 
+    if config.LOGSTASH_SERVER and not len(config.LOGSTASH_SERVER.split(':')) == 2:
+        exit("[!] invalid configuration value for 'LOGSTASH_SERVER' ('%s')" % config.LOGSTASH_SERVER)
+
+    if config.REMOTE_SEVERITY_REGEX:
+        try:
+            re.compile(config.REMOTE_SEVERITY_REGEX)
+        except re.error:
+            exit("[!] invalid configuration value for 'REMOTE_SEVERITY_REGEX' ('%s')" % config.REMOTE_SEVERITY_REGEX)
+
     if config.CAPTURE_FILTER:
         print("[i] setting capture filter '%s'" % config.CAPTURE_FILTER)
         for _cap in _caps:
@@ -992,6 +1009,7 @@ def _init_multiprocessing():
     """
 
     global _buffer
+    global _multiprocessing
     global _n
 
     if _multiprocessing:
@@ -1008,13 +1026,18 @@ def _init_multiprocessing():
         except:
             exit("[!] unable to allocate network capture buffer. Please adjust value of 'CAPTURE_BUFFER'")
 
-        print("[i] creating %d more processes (out of total %d)" % (config.PROCESS_COUNT - 1, config.PROCESS_COUNT))
         _n = _multiprocessing.Value('L', lock=False)
 
-        for i in xrange(config.PROCESS_COUNT - 1):
-            process = _multiprocessing.Process(target=worker, name=str(i), args=(_buffer, _n, i, config.PROCESS_COUNT - 1, _process_packet))
-            process.daemon = True
-            process.start()
+        try:
+            for i in xrange(config.PROCESS_COUNT - 1):
+                process = _multiprocessing.Process(target=worker, name=str(i), args=(_buffer, _n, i, config.PROCESS_COUNT - 1, _process_packet))
+                process.daemon = True
+                process.start()
+        except TypeError:   # Note: https://github.com/stamparm/maltrail/issues/11823
+            _buffer = None
+            _multiprocessing = None
+        else:
+            print("[i] created %d more processes (out of total %d)" % (config.PROCESS_COUNT - 1, config.PROCESS_COUNT))
 
 def monitor():
     """
@@ -1086,7 +1109,7 @@ def monitor():
 
             datalink = _cap.datalink()
 
-            if six.PY3:  # https://github.com/helpsystems/pcapy/issues/37#issuecomment-530795813
+            if six.PY3 and not config.pcap_file:  # https://github.com/helpsystems/pcapy/issues/37#issuecomment-530795813
                 def _loop_handler(header, packet):
                     packet_handler(datalink, header, packet)
 
