@@ -271,11 +271,42 @@ $( document ).ready(function() {
       $(".rule_select").prop("checked", $(this).prop("checked"));
   });
 
+  // move category block
+  $("#category_block").detach().appendTo($(".page-content-head > .container-fluid > .list-inline"));
+  $("#category_block").addClass("pull-right");
+
+  // our usual zebra striping doesn't respect hidden rows, hook repaint on .opnsense-rules change() and fire initially
+  $(".opnsense-rules > tbody > tr").each(function(){
+      // save zebra color
+      let tr_color = $(this).children(0).css("background-color");
+      if (tr_color != 'transparent' && !tr_color.includes('(0, 0, 0')) {
+          $("#fw_category").data('stripe_color', tr_color);
+      }
+  });
+  $(".opnsense-rules").removeClass("table-striped");
+  $(".opnsense-rules").change(function(){
+      $(".opnsense-rules > tbody > tr:visible").each(function (index) {
+          $(this).css("background-color", "inherit");
+          if ( index % 2 == 0) {
+              $(this).css("background-color", $("#fw_category").data('stripe_color'));
+          }
+      });
+  });
+
+  // hook category functionality
+  hook_firewall_categories();
+
   // watch scroll position and set to last known on page load
   watchScrollPosition();
 });
 </script>
 <?php include("fbegin.inc"); ?>
+  <div class="hidden">
+    <div id="category_block" style="z-index:-100;">
+        <select class="selectpicker hidden-xs hidden-sm hidden-md" data-live-search="true" data-size="5"  multiple title="<?=gettext("Select category");?>" id="fw_category">
+        </select>
+    </div>
+  </div>
   <section class="page-content-main">
     <div class="container-fluid">
       <div class="row">
@@ -290,7 +321,7 @@ $( document ).ready(function() {
               <input type="hidden" id="id" name="id" value="" />
               <input type="hidden" id="action" name="act" value="" />
               <div class="table-responsive">
-                <table class="table table-striped">
+                <table class="table table-striped table-condensed opnsense-rules">
                   <thead>
                     <tr>
                       <td colspan="5"> </td>
@@ -303,7 +334,7 @@ $( document ).ready(function() {
                       <th style="width:2%"><input type="checkbox" id="selectAll"></th>
                       <th style="width:2%">&nbsp;</th>
                       <th style="width:2%">&nbsp;</th>
-                      <th><?=gettext("If");?></th>
+                      <th><?=gettext("Interface");?></th>
                       <th><?=gettext("Proto");?></th>
                       <th class="hidden-xs hidden-sm"><?=gettext("Address");?></th>
                       <th class="hidden-xs hidden-sm"><?=gettext("Ports");?></th>
@@ -338,7 +369,7 @@ $( document ).ready(function() {
 <?php               $nnats = 0;
                     foreach ($a_nat as $natent):
 ?>
-                    <tr <?=isset($natent['disabled'])?"class=\"text-muted\"":"";?> ondblclick="document.location='firewall_nat_edit.php?id=<?=$nnats;?>';">
+                    <tr class="rule <?=isset($natent['disabled'])?"text-muted":"";?>" data-category="<?=!empty($natent['category']) ? $natent['category'] : "";?>" ondblclick="document.location='firewall_nat_edit.php?id=<?=$nnats;?>';">
                       <td>
                         <input class="rule_select" type="checkbox" name="rule[]" value="<?=$nnats;?>"  />
                       </td>
@@ -363,7 +394,11 @@ $( document ).ready(function() {
                         </a>
                       </td>
                       <td>
-                        <?=htmlspecialchars(convert_friendly_interface_to_friendly_descr(isset($natent['interface']) ? $natent['interface'] : "wan"));?>
+<?php
+                          foreach (explode(",", $natent['interface']) as $intf):?>
+                              <?=htmlspecialchars(convert_friendly_interface_to_friendly_descr($intf));?>
+<?php
+                          endforeach;?>
                       </td>
                       <td>
                         <?=strtoupper($natent['protocol']);?>
@@ -458,7 +493,7 @@ $( document ).ready(function() {
 <?php                   endif; ?>
                       </td>
 
-                      <td>
+                      <td class="rule-description">
                         <?=$natent['descr'];?>
                       </td>
 
