@@ -1,41 +1,42 @@
 <?php
 
 /*
-    Copyright (c) 2015 Franco Fichtner <franco@opnsense.org>
-    Copyright (c) 2008 Tellnet AG
-    All rights reserved.
-
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
-
-    1. Redistributions of source code must retain the above copyright notice,
-       this list of conditions and the following disclaimer.
-
-    2. Redistributions in binary form must reproduce the above copyright
-       notice, this list of conditions and the following disclaimer in the
-       documentation and/or other materials provided with the distribution.
-
-    THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
-    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-    AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
-    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
-*/
+ * Copyright (c) 2015-2021 Franco Fichtner <franco@opnsense.org>
+ * Copyright (c) 2008 Tellnet AG
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ */
 
 require_once("guiconfig.inc");
 require_once("system.inc");
-require_once("services.inc");
 require_once("interfaces.inc");
+require_once("plugins.inc.d/opendns.inc");
 
 config_read_array('opendns');
 
 if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     $pconfig['enable'] = isset($config['opendns']['enable']);
+    $pconfig['standalone'] = isset($config['opendns']['standalone']);
     $pconfig['username'] = !empty($config['opendns']['username']) ? $config['opendns']['username'] : null;
     $pconfig['password'] = !empty($config['opendns']['password']) ? $config['opendns']['password'] : null;
     $pconfig['host'] = !empty($config['opendns']['host']) ? $config['opendns']['host'] : null;
@@ -60,19 +61,16 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     }
 
     if (!empty($pconfig['test'])) {
-        $ch = curl_init();
-        curl_setopt($ch, CURLOPT_URL, sprintf( 'https://updates.opendns.com/nic/update?hostname=%s', $pconfig['host']));
-        curl_setopt($ch, CURLOPT_USERPWD, sprintf('%s:%s', $pconfig['username'], $pconfig['password']));
-        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
-        $output = curl_exec($ch);
-        curl_close($ch);
-        $test_results = explode("\r\n", $output);
+        $test_results = explode("\r\n", opendns_register($pconfig));
     } elseif (count($input_errors) == 0) {
         $config['opendns']['enable'] = !empty($pconfig['enable']);
+        $config['opendns']['standalone'] = !empty($pconfig['standalone']);
         $config['opendns']['username'] = $pconfig['username'];
         $config['opendns']['password'] = $pconfig['password'];
         $config['opendns']['host'] = $pconfig['host'];
-        if ($config['opendns']['enable']) {
+        if ($config['opendns']['standalone']) {
+            /* nothing to do, keep system state */
+        } elseif ($config['opendns']['enable']) {
             $config['system']['dnsserver'] = array();
             $v4_server = array('208.67.222.222', '208.67.220.220');
             $v6_server = array('2620:0:ccc::2', '2620:0:ccd::2');
@@ -101,7 +99,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         }
         write_config('OpenDNS filter configuration change');
         system_resolvconf_generate();
-        services_dhcpd_configure();
+        plugins_configure('dhcp');
         $savemsg = get_std_save_message();
     }
 }
@@ -143,7 +141,7 @@ include 'head.inc';
                   <td><a id="help_for_enable" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext('Enable'); ?></td>
                   <td>
                     <input name="enable" type="checkbox" id="enable" value="yes" <?=!empty($pconfig['enable']) ? 'checked="checked"' : "";?> />
-                    <strong><?=gettext('Filter DNS requests using OpenDNS'); ?></strong>
+                    <?= gettext('Filter DNS requests using OpenDNS') ?>
                     <div class="hidden" data-for="help_for_enable">
                       <?= sprintf(gettext(
                         'Enabling the OpenDNS service will overwrite DNS servers configured ' .
@@ -154,6 +152,19 @@ include 'head.inc';
                     </div>
                   </td>
                 </tr>
+                <tr>
+                  <td><a id="help_for_standalone" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext('Standalone'); ?></td>
+                  <td>
+                    <input name="standalone" type="checkbox" id="standalone" value="yes" <?=!empty($pconfig['standalone']) ? 'checked="checked"' : "";?> />
+                    <?= gettext('Do not alter system DNS server settings') ?>
+                    <div class="hidden" data-for="help_for_standalone">
+                      <?= sprintf(gettext(
+                        'Enable this mode when the OpenDNS servers are used by a different ' .
+                        'network component but the periodic update behaviour is still desired.'
+                      )) ?>
+                    </div>
+                  </td>
+                </tr>
                 <tr>
                   <td><a id="help_for_username" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext('Username'); ?></td>
                   <td>
