@@ -386,6 +386,8 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             local_user_set_groups($userent, $pconfig['groups']);
             local_user_set($userent);
             write_config();
+            // XXX: signal backend that the user has changed.
+            configdp_run('auth user changed', [$userent['name']]);
 
             if (!empty($pconfig['chkNewCert'])) {
                 header(url_safe('Location: /system_certmanager.php?act=new&userid=%d', array(isset($id) ? $id : count($a_user) - 1)));
@@ -484,7 +486,7 @@ $( document ).ready(function() {
       event.preventDefault();
       const url="system_usermanager_import_ldap.php";
       var oWin = window.open(url,"OPNsense","width=620,height=400,top=150,left=150,scrollbars=yes");
-      if (oWin==null || typeof(oWin)=="undefined") {
+      if (oWin == null || typeof(oWin) == "undefined") {
         alert("<?= html_safe(gettext('Popup blocker detected. Action aborted.')) ?>");
       }
     });
