@@ -31,14 +31,13 @@
 require_once("guiconfig.inc");
 require_once("interfaces.inc");
 require_once("plugins.inc.d/ipsec.inc");
-require_once("services.inc");
 
 /**
  * combine ealgos and keylen_* tags
  */
 function pconfig_to_ealgos($pconfig)
 {
-    global $p2_ealgos;
+    $p2_ealgos = ipsec_p2_ealgos();
 
     $ealgos = array();
     if (isset($pconfig['ealgos'])) {
@@ -192,7 +191,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
 
         /* mobile client */
         if (isset($_GET['mobile'])) {
-            $pconfig['mobile']=true;
+            $pconfig['mobile'] = true;
         }
         // init empty
         foreach (explode(",", $phase2_fields) as $fieldname) {
@@ -272,27 +271,19 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         }
     } elseif ($pconfig['mode'] == 'route-based') {
         // validate if both tunnel networks are using the correct address family
-        $protocol = 'inet';
-        foreach ($config['ipsec']['phase1'] as $phase1ent) {
-            if ($phase1ent['ikeid'] == $pconfig['ikeid']) {
-                $protocol = $phase1ent['protocol'];
-                break;
-            }
-        }
-        if ($protocol == 'inet') {
-            if (!is_ipaddrv4($pconfig['tunnel_local'])) {
+        if (!is_ipaddr($pconfig['tunnel_local']) || !is_ipaddr($pconfig['tunnel_remote'])) {
+            if (!is_ipaddr($pconfig['tunnel_local'])) {
                 $input_errors[] = gettext('A valid local network IP address must be specified.');
             }
-            if (!is_ipaddrv4($pconfig['tunnel_remote'])) {
-                $input_errors[] = gettext("A valid remote network IP address must be specified.");
-            }
-        } else {
-            if (!is_ipaddrv6($pconfig['tunnel_local'])) {
-                $input_errors[] = gettext('A valid local network IP address must be specified.');
-            }
-            if (!is_ipaddrv6($pconfig['tunnel_remote'])) {
+            if (!is_ipaddr($pconfig['tunnel_remote'])) {
                 $input_errors[] = gettext("A valid remote network IP address must be specified.");
             }
+        } elseif(
+            !(is_ipaddrv4($pconfig['tunnel_local']) && is_ipaddrv4($pconfig['tunnel_remote'])) &&
+            !(is_ipaddrv6($pconfig['tunnel_local']) && is_ipaddrv6($pconfig['tunnel_remote']))
+        ) {
+            $input_errors[] = gettext('A valid local network IP address must be specified.');
+            $input_errors[] = gettext("A valid remote network IP address must be specified.");
         }
     }
     /* Validate enabled phase2's are not duplicates */
@@ -416,7 +407,23 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         if (isset($pconfig['mobile'])) {
             $ph2ent['mobile'] = true;
         }
-
+        // attach or generate reqid
+        if ($p2index !== null && !empty($config['ipsec']['phase2'][$p2index]['reqid'])) {
+            $ph2ent['reqid'] = $config['ipsec']['phase2'][$p2index]['reqid'];
+        } else {
+            $reqids = [];
+            foreach ($config['ipsec']['phase2'] as $tmp) {
+                if (!empty($tmp['reqid'])) {
+                    $reqids[] = $tmp['reqid'];
+                }
+            }
+            for ($i=1; $i < 65535; $i++) {
+                if (!in_array($i, $reqids)) {
+                    $ph2ent['reqid'] = $i;
+                    break;
+                }
+            }
+        }
         // save to config
         if ($p2index !== null) {
             $config['ipsec']['phase2'][$p2index] = $ph2ent;
@@ -632,9 +639,9 @@ if (isset($input_errors) && count($input_errors) > 0) {
                 <tr class="opt_remoteid">
                   <td><i class="fa fa-info-circle text-muted"></i> <?=gettext("Address"); ?>:&nbsp;&nbsp;</td>
                   <td>
-                    <input name="remoteid_address" type="text" class="formfld unknown ipv4v6" id="remoteid_address" size="28" value="<?=$pconfig['remoteid_address'];?>" />
+                    <input name="remoteid_address" type="text" class="formfld unknown" id="remoteid_address" size="28" value="<?=$pconfig['remoteid_address'];?>" />
                     /
-                    <select name="remoteid_netbits" class="ipv4v6" id="remoteid_netbits">
+                    <select name="remoteid_netbits" data-network-id="remoteid_address" class="ipv4v6net" id="remoteid_netbits">
 <?php              for ($i = 128; $i >= 0; $i--) :
 ?>
                       <option value="<?=$i;?>" <?= isset($pconfig['remoteid_netbits']) && $i == $pconfig['remoteid_netbits'] ? "selected=\"selected\"" : "";?> >
@@ -675,7 +682,7 @@ endif; ?>
                   <td><a id="help_for_encalg" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Encryption algorithms"); ?></td>
                   <td>
 <?php
-                  foreach ($p2_ealgos as $algo => $algodata) :?>
+                  foreach (ipsec_p2_ealgos() as $algo => $algodata) :?>
                     <input type="checkbox" name="ealgos[]" value="<?=$algo;?>" <?=isset($pconfig['ealgos']) && in_array($algo, $pconfig['ealgos']) ? "checked=\"checked\"" : ""; ?> />
                       <?=$algodata['name'];?>
 <?php
@@ -710,8 +717,8 @@ endif; ?>
                   <td><i class="fa fa-info-circle text-muted"></i> <?=gettext("Hash algorithms"); ?></td>
                   <td style="width:78%" class="vtable">
                     <select name="hash-algorithm-option[]" class="selectpicker" multiple="multiple">
-<?php foreach ($p2_halgos as $algo => $algoname): ?>
-                      <option value="<?= html_safe($algo) ?>" <?= in_array($algo, $pconfig['hash-algorithm-option']) ? 'selected="selected"' : '' ?>>
+<?php foreach (ipsec_p2_halgos() as $algo => $algoname): ?>
+                      <option value="<?= html_safe($algo) ?>" <?= (is_array($pconfig['hash-algorithm-option']) && in_array($algo, $pconfig['hash-algorithm-option'])) ? 'selected="selected"' : '' ?>>
                         <?= html_safe($algoname) ?>
                       </option>
 <?php endforeach ?>
@@ -744,6 +751,7 @@ endif; ?>
                         28 => '28 (Brainpool EC 256 bits)',
                         29 => '29 (Brainpool EC 384 bits)',
                         30 => '30 (Brainpool EC 512 bits)',
+                        31 => '31 (Elliptic Curve 25519)',
                     );
                     foreach ($p2_dhgroups as $keygroup => $keygroupname): ?>
                       <option value="<?=$keygroup;?>" <?= $keygroup == $pconfig['pfsgroup'] ? "selected=\"selected\"" : "";?>>
